@@ -1,4 +1,4 @@
-/* $Id: cmd-command-prompt.c,v 1.9 2009/01/14 22:16:56 nicm Exp $ */
+/* $Id: cmd-command-prompt.c,v 1.13 2009/01/30 21:10:10 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,7 +28,7 @@
  */
 
 void	cmd_command_prompt_init(struct cmd *, int);
-void	cmd_command_prompt_exec(struct cmd *, struct cmd_ctx *);
+int	cmd_command_prompt_exec(struct cmd *, struct cmd_ctx *);
 
 int	cmd_command_prompt_callback(void *, const char *);
 
@@ -60,12 +60,18 @@ cmd_command_prompt_init(struct cmd *self, int key)
 
 	switch (key) {
 	case ',':
-		data->arg = xstrdup("rename-window \"%%\"");
+		data->arg = xstrdup("rename-window '%%'");
+		break;
+	case '.':
+		data->arg = xstrdup("move-window -t '%%'");
+		break;
+	case 'f':
+		data->arg = xstrdup("find-window '%%'");
 		break;
 	}
 }
 
-void
+int
 cmd_command_prompt_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_target_data		*data = self->data;
@@ -74,10 +80,10 @@ cmd_command_prompt_exec(struct cmd *self, struct cmd_ctx *ctx)
 	char				*hdr, *ptr;
 
 	if ((c = cmd_find_client(ctx, data->target)) == NULL)
-		return;
+		return (-1);
 
 	if (c->prompt_string != NULL)
-		return;
+		return (0);
 
 	cdata = xmalloc(sizeof *cdata);
 	cdata->c = c;
@@ -93,8 +99,7 @@ cmd_command_prompt_exec(struct cmd *self, struct cmd_ctx *ctx)
 	server_set_client_prompt(c, hdr, cmd_command_prompt_callback, cdata, 0);
 	xfree(hdr);
 
-	if (ctx->cmdclient != NULL)
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+	return (0);
 }
 
 int
@@ -102,7 +107,7 @@ cmd_command_prompt_callback(void *data, const char *s)
 {
 	struct cmd_command_prompt_data	*cdata = data;
 	struct client			*c = cdata->c;
-	struct cmd			*cmd;
+	struct cmd_list			*cmdlist;
 	struct cmd_ctx	 		 ctx;
 	char				*cause, *ptr, *buf, ch;
 	size_t				 len, slen;
@@ -139,17 +144,17 @@ cmd_command_prompt_callback(void *data, const char *s)
 	}
 	xfree(cdata);
 
-	if (cmd_string_parse(s, &cmd, &cause) != 0) {
+	if (cmd_string_parse(s, &cmdlist, &cause) != 0) {
 		if (cause == NULL)
 			return (0);
 		*cause = toupper((u_char) *cause);
 		server_set_client_message(c, cause);
 		xfree(cause);
-		cmd = NULL;
+		cmdlist = NULL;
 	}
 	if (buf != NULL)
 		xfree(buf);
-	if (cmd == NULL)
+	if (cmdlist == NULL)
 		return (0);
 
 	ctx.msgdata = NULL;
@@ -162,7 +167,7 @@ cmd_command_prompt_callback(void *data, const char *s)
 
 	ctx.cmdclient = NULL;
 
-	cmd_exec(cmd, &ctx);
+	cmd_list_exec(cmdlist, &ctx);
 
 	if (c->prompt_callback != (void *) &cmd_command_prompt_callback)
 		return (1);
