@@ -1,4 +1,4 @@
-/* $Id: cmd-copy-mode.c,v 1.13 2009/01/11 23:31:46 nicm Exp $ */
+/* $Id: cmd-copy-mode.c,v 1.15 2009/01/27 23:35:44 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,12 +24,12 @@
  * Enter copy mode.
  */
 
-void	cmd_copy_mode_exec(struct cmd *, struct cmd_ctx *);
+int	cmd_copy_mode_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_copy_mode_entry = {
 	"copy-mode", NULL,
 	CMD_TARGET_WINDOW_USAGE,
-	0,
+	CMD_UFLAG,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_copy_mode_exec,
@@ -39,17 +39,19 @@ const struct cmd_entry cmd_copy_mode_entry = {
 	NULL
 };
 
-void
+int
 cmd_copy_mode_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_target_data	*data = self->data;
 	struct winlink		*wl;
 
 	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
-		return;
+		return (-1);
 
 	window_pane_set_mode(wl->window->active, &window_copy_mode);
+	if (data->flags & CMD_UFLAG)
+		window_copy_pageup(wl->window->active);
 
-	if (ctx->cmdclient != NULL)
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+
+	return (0);
 }
