@@ -1,4 +1,4 @@
-/* $Id: cmd-new-window.c,v 1.27 2009/01/10 19:37:35 nicm Exp $ */
+/* $Id: cmd-new-window.c,v 1.31 2009/01/23 16:59:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -27,23 +27,24 @@
  */
 
 int	cmd_new_window_parse(struct cmd *, int, char **, char **);
-void	cmd_new_window_exec(struct cmd *, struct cmd_ctx *);
+int	cmd_new_window_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_new_window_send(struct cmd *, struct buffer *);
 void	cmd_new_window_recv(struct cmd *, struct buffer *);
 void	cmd_new_window_free(struct cmd *);
 void	cmd_new_window_init(struct cmd *, int);
-void	cmd_new_window_print(struct cmd *, char *, size_t);
+size_t	cmd_new_window_print(struct cmd *, char *, size_t);
 
 struct cmd_new_window_data {
 	char	*target;
 	char	*name;
 	char	*cmd;
 	int	 flag_detached;
+	int	 flag_kill;
 };
 
 const struct cmd_entry cmd_new_window_entry = {
 	"new-window", "neww",
-	"[-d] [-n window-name] [-t target-window] [command]",
+	"[-dk] [-n window-name] [-t target-window] [command]",
 	0,
 	cmd_new_window_init,
 	cmd_new_window_parse,
@@ -64,6 +65,7 @@ cmd_new_window_init(struct cmd *self, unused int arg)
 	data->name = NULL;
 	data->cmd = NULL;
 	data->flag_detached = 0;
+	data->flag_kill = 0;
 }
 
 int
@@ -75,11 +77,14 @@ cmd_new_window_parse(struct cmd *self, int argc, char **argv, char **cause)
 	self->entry->init(self, 0);
 	data = self->data;
 
-	while ((opt = getopt(argc, argv, "dt:n:")) != -1) {
+	while ((opt = getopt(argc, argv, "dkt:n:")) != -1) {
 		switch (opt) {
 		case 'd':
 			data->flag_detached = 1;
 			break;
+		case 'k':
+			data->flag_kill = 1;
+			break;
 		case 't':
 			if (data->target == NULL)
 				data->target = xstrdup(optarg);
@@ -109,21 +114,21 @@ usage:
 	return (-1);
 }
 
-void
+int
 cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_new_window_data	*data = self->data;
 	struct session			*s;
 	struct winlink			*wl;
-	char				*cmd, *cwd;
+	char				*cmd, *cwd, *cause;
 	int				 idx;
 
 	if (data == NULL)
-		return;
+		return (0);
 
 	if (arg_parse_window(data->target, &s, &idx) != 0) {
 		ctx->error(ctx, "bad window: %s", data->target);
-		return;
+		return (-1);
 	}
 	if (s == NULL)
 		s = ctx->cursession;
@@ -131,7 +136,28 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		s = cmd_current_session(ctx);
 	if (s == NULL) {
 		ctx->error(ctx, "session not found: %s", data->target);
-		return;
+		return (-1);
+	}
+
+	wl = NULL;
+	if (idx != -1)
+		wl = winlink_find_by_index(&s->windows, idx);
+	if (wl != NULL) {
+		if (data->flag_kill) {
+			/*
+			 * Can't use session_detach as it will destroy session
+			 * if this makes it empty.
+			 */
+			session_alert_cancel(s, wl);
+			winlink_stack_remove(&s->lastw, wl);
+			winlink_remove(&s->windows, wl);
+
+			/* Force select/redraw if current. */
+			if (wl == s->curw) {
+				data->flag_detached = 0;
+				s->curw = NULL;
+			}
+		}
 	}
 
 	cmd = data->cmd;
@@ -142,10 +168,11 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	else
 		cwd = ctx->cmdclient->cwd;
 
-	wl = session_new(s, data->name, cmd, cwd, idx);
+	wl = session_new(s, data->name, cmd, cwd, idx, &cause);
 	if (wl == NULL) {
-		ctx->error(ctx, "command failed: %s", cmd);
-		return;
+		ctx->error(ctx, "create window failed: %s", cause);
+		xfree(cause);
+		return (-1);
 	}
 	if (!data->flag_detached) {
 		session_select(s, wl->idx);
@@ -153,8 +180,7 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	} else
 		server_status_session(s);
 
-	if (ctx->cmdclient != NULL)
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+	return (0);
 }
 
 void
@@ -194,7 +220,7 @@ cmd_new_window_free(struct cmd *self)
 	xfree(data);
 }
 
-void
+size_t
 cmd_new_window_print(struct cmd *self, char *buf, size_t len)
 {
 	struct cmd_new_window_data	*data = self->data;
@@ -202,13 +228,14 @@ cmd_new_window_print(struct cmd *self, char *buf, size_t len)
 
 	off += xsnprintf(buf, len, "%s", self->entry->name);
 	if (data == NULL)
-		return;
+		return (off);
 	if (off < len && data->flag_detached)
 		off += xsnprintf(buf + off, len - off, " -d");
 	if (off < len && data->target != NULL)
-		off += xsnprintf(buf + off, len - off, " -t %s", data->target);
+		off += cmd_prarg(buf + off, len - off, " -t ", data->target);
 	if (off < len && data->name != NULL)
-		off += xsnprintf(buf + off, len - off, " -n %s", data->name);
+		off += cmd_prarg(buf + off, len - off, " -n ", data->name);
 	if (off < len && data->cmd != NULL)
-		off += xsnprintf(buf + off, len - off, " %s", data->cmd);
+		off += cmd_prarg(buf + off, len - off, " ", data->cmd);
+	return (off);
 }
