@@ -1,4 +1,4 @@
-/* $Id: cmd-scroll-mode.c,v 1.14 2009/01/11 23:31:46 nicm Exp $ */
+/* $Id: cmd-scroll-mode.c,v 1.16 2009/01/27 23:35:44 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,13 +24,14 @@
  * Enter scroll mode.
  */
 
-void	cmd_scroll_mode_exec(struct cmd *, struct cmd_ctx *);
+void	cmd_scroll_mode_init(struct cmd *, int);
+int	cmd_scroll_mode_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_scroll_mode_entry = {
 	"scroll-mode", NULL,
 	CMD_TARGET_WINDOW_USAGE,
-	0,
-	cmd_target_init,
+	CMD_UFLAG,
+	cmd_scroll_mode_init,
 	cmd_target_parse,
 	cmd_scroll_mode_exec,
 	cmd_target_send,
@@ -40,16 +41,32 @@ const struct cmd_entry cmd_scroll_mode_entry = {
 };
 
 void
+cmd_scroll_mode_init(struct cmd *self, int key)
+{
+	struct cmd_target_data	*data;
+
+	cmd_target_init(self, key);
+	data = self->data;
+
+	switch (key) {
+	case KEYC_PPAGE:
+		data->flags |= CMD_UFLAG;
+		break;
+	}
+}
+
+int
 cmd_scroll_mode_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_target_data	*data = self->data;
 	struct winlink		*wl;
 
 	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
-		return;
+		return (-1);
 
 	window_pane_set_mode(wl->window->active, &window_scroll_mode);
+	if (data->flags & CMD_UFLAG)
+		window_scroll_pageup(wl->window->active);
 
-	if (ctx->cmdclient != NULL)
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+	return (0);
 }
