@@ -1,4 +1,4 @@
-/* $Id: cmd-split-window.c,v 1.4 2009/01/14 19:29:32 nicm Exp $ */
+/* $Id: cmd-split-window.c,v 1.8 2009/01/23 16:59:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,22 +28,24 @@
  */
 
 int	cmd_split_window_parse(struct cmd *, int, char **, char **);
-void	cmd_split_window_exec(struct cmd *, struct cmd_ctx *);
+int	cmd_split_window_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_split_window_send(struct cmd *, struct buffer *);
 void	cmd_split_window_recv(struct cmd *, struct buffer *);
 void	cmd_split_window_free(struct cmd *);
 void	cmd_split_window_init(struct cmd *, int);
-void	cmd_split_window_print(struct cmd *, char *, size_t);
+size_t	cmd_split_window_print(struct cmd *, char *, size_t);
 
 struct cmd_split_window_data {
 	char	*target;
 	char	*cmd;
 	int	 flag_detached;
+	int	 percentage;
+	int	 lines;
 };
 
 const struct cmd_entry cmd_split_window_entry = {
 	"split-window", "splitw",
-	"[-d] [-t target-window] [command]",
+	"[-d] [-p percentage|-l lines] [-t target-window] [command]",
 	0,
 	cmd_split_window_init,
 	cmd_split_window_parse,
@@ -63,13 +65,16 @@ cmd_split_window_init(struct cmd *self, unused int arg)
 	data->target = NULL;
 	data->cmd = NULL;
 	data->flag_detached = 0;
+	data->percentage = -1;
+	data->lines = -1;
 }
 
 int
 cmd_split_window_parse(struct cmd *self, int argc, char **argv, char **cause)
 {
 	struct cmd_split_window_data	*data;
-	int				 opt;
+	int				 opt, n;
+	const char			*errstr;
 
 	self->entry->init(self, 0);
 	data = self->data;
@@ -83,6 +88,27 @@ cmd_split_window_parse(struct cmd *self, int argc, char **argv, char **cause)
 			if (data->target == NULL)
 				data->target = xstrdup(optarg);
 			break;
+		case 'l':
+			if (data->percentage == -1 && data->lines == -1) {
+				n = strtonum(optarg, 1, INT_MAX, &errstr);
+				if (errstr != NULL) {
+					xasprintf(cause, "lines %s", errstr);
+					goto error;
+				}
+				data->lines = n;
+			}
+			break;
+		case 'p':
+			if (data->lines == -1 && data->percentage == -1) {
+				n = strtonum(optarg, 1, 100, &errstr);
+				if (errstr != NULL) {
+					xasprintf(
+					    cause, "percentage %s", errstr);
+					goto error;
+				}
+				data->percentage = n;
+			}
+			break;
 		default:
 			goto usage;
 		}
@@ -100,11 +126,12 @@ cmd_split_window_parse(struct cmd *self, int argc, char **argv, char **cause)
 usage:
 	xasprintf(cause, "usage: %s %s", self->entry->name, self->entry->usage);
 
+error:
 	self->entry->free(self);
 	return (-1);
 }
 
-void
+int
 cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_split_window_data	*data = self->data;
@@ -113,12 +140,11 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct window			*w;
 	struct window_pane		*wp;
 	const char			*env[] = CHILD_ENVIRON;
-	char		 		 buf[256];
-	char				*cmd, *cwd;
-	u_int				 i, hlimit;
+	char		 		 buf[256], *cmd, *cwd, *cause;
+	u_int				 i, hlimit, lines;
 
 	if ((wl = cmd_find_window(ctx, data->target, &s)) == NULL)
-		return;
+		return (-1);
 	w = wl->window;
 
 	if (session_index(s, &i) != 0)
@@ -134,10 +160,18 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	else
 		cwd = ctx->cmdclient->cwd;
 
+	lines = -1;
+	if (data->lines != -1)
+		lines = data->lines;
+	else if (data->percentage != -1)
+		lines = (w->active->sy * data->percentage) / 100;
+
 	hlimit = options_get_number(&s->options, "history-limit");
-	if ((wp = window_add_pane(w, cmd, cwd, env, hlimit)) == NULL) {
-		ctx->error(ctx, "command failed: %s", cmd);
-		return;
+	wp = window_add_pane(w, lines, cmd, cwd, env, hlimit, &cause);
+	if (wp == NULL) {
+		ctx->error(ctx, "create pane failed: %s", cause);
+		xfree(cause);
+		return (-1);
 	}
 	server_redraw_window(w);
 	
@@ -148,8 +182,7 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	} else
 		server_status_session(s);
 
-	if (ctx->cmdclient != NULL)
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+	return (0);
 }
 
 void
@@ -185,7 +218,7 @@ cmd_split_window_free(struct cmd *self)
 	xfree(data);
 }
 
-void
+size_t
 cmd_split_window_print(struct cmd *self, char *buf, size_t len)
 {
 	struct cmd_split_window_data	*data = self->data;
@@ -193,11 +226,12 @@ cmd_split_window_print(struct cmd *self, char *buf, size_t len)
 
 	off += xsnprintf(buf, len, "%s", self->entry->name);
 	if (data == NULL)
-		return;
+		return (off);
 	if (off < len && data->flag_detached)
 		off += xsnprintf(buf + off, len - off, " -d");
 	if (off < len && data->target != NULL)
-		off += xsnprintf(buf + off, len - off, " -t %s", data->target);
+		off += cmd_prarg(buf + off, len - off, " -t ", data->target);
 	if (off < len && data->cmd != NULL)
-		off += xsnprintf(buf + off, len - off, " %s", data->cmd);
+		off += cmd_prarg(buf + off, len - off, " ", data->cmd);
+	return (off);
 }
