@@ -1,4 +1,4 @@
-/* $Id: key-bindings.c,v 1.53 2009/01/17 18:34:12 nicm Exp $ */
+/* $Id: key-bindings.c,v 1.59 2009/01/30 21:10:10 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -44,7 +44,7 @@ key_bindings_lookup(int key)
 }
 
 void
-key_bindings_add(int key, struct cmd *cmd)
+key_bindings_add(int key, struct cmd_list *cmdlist)
 {
 	struct key_binding	*bd;
 
@@ -53,8 +53,8 @@ key_bindings_add(int key, struct cmd *cmd)
 		bd->key = key;
 		SPLAY_INSERT(key_bindings, &key_bindings, bd);
 	} else
-		cmd_free(bd->cmd);
-	bd->cmd = cmd;
+		cmd_list_free(bd->cmdlist);
+	bd->cmdlist = cmdlist;
 }
 
 void
@@ -66,7 +66,7 @@ key_bindings_remove(int key)
 		return;
 	SPLAY_REMOVE(key_bindings, &key_bindings, bd);
 
-	cmd_free(bd->cmd);
+	cmd_list_free(bd->cmdlist);
 	xfree(bd);
 }
 
@@ -81,6 +81,7 @@ key_bindings_init(void)
 		{ '#', &cmd_list_buffers_entry },
 		{ '&', &cmd_kill_window_entry },
 		{ ',', &cmd_command_prompt_entry },
+		{ '.', &cmd_command_prompt_entry },
 		{ '-', &cmd_delete_buffer_entry },
 		{ '0', &cmd_select_window_entry },
 		{ '1', &cmd_select_window_entry },
@@ -100,6 +101,7 @@ key_bindings_init(void)
 		{ ']', &cmd_paste_buffer_entry },
 		{ 'c', &cmd_new_window_entry },
 		{ 'd', &cmd_detach_client_entry },
+		{ 'f', &cmd_command_prompt_entry },
 		{ 'l', &cmd_last_window_entry },
 		{ 'n', &cmd_next_window_entry },
 		{ 'o', &cmd_down_pane_entry },
@@ -108,7 +110,11 @@ key_bindings_init(void)
 		{ 's', &cmd_choose_session_entry },
 		{ 't', &cmd_clock_mode_entry },
 		{ 'w', &cmd_choose_window_entry },
-		{ 'x', &cmd_kill_pane_entry, },	
+		{ 'x', &cmd_kill_pane_entry, },
+		{ '\032', &cmd_suspend_client_entry },
+		{ KEYC_PPAGE, &cmd_scroll_mode_entry },
+		{ KEYC_ADDESC('n'), &cmd_next_window_entry },
+		{ KEYC_ADDESC('p'), &cmd_previous_window_entry },
 		{ KEYC_UP, &cmd_up_pane_entry },
 		{ KEYC_DOWN, &cmd_down_pane_entry },
 		{ KEYC_ADDESC(KEYC_UP), &cmd_resize_pane_up_entry },
@@ -119,16 +125,22 @@ key_bindings_init(void)
 	};
 	u_int		 i;
 	struct cmd	*cmd;
+	struct cmd_list	*cmdlist;
 
 	SPLAY_INIT(&key_bindings);
 
 	for (i = 0; i < nitems(table); i++) {
+		cmdlist = xmalloc(sizeof *cmdlist);
+		TAILQ_INIT(cmdlist);
+
 		cmd = xmalloc(sizeof *cmd);
 		cmd->entry = table[i].entry;
 		cmd->data = NULL;
 		if (cmd->entry->init != NULL)
 			cmd->entry->init(cmd, table[i].key);
-		key_bindings_add(table[i].key, cmd);
+		TAILQ_INSERT_HEAD(cmdlist, cmd, qentry);
+
+		key_bindings_add(table[i].key, cmdlist);
 	}
 }
 
@@ -140,7 +152,7 @@ key_bindings_free(void)
 	while (!SPLAY_EMPTY(&key_bindings)) {
 		bd = SPLAY_ROOT(&key_bindings);
 		SPLAY_REMOVE(key_bindings, &key_bindings, bd);
-		cmd_free(bd->cmd);
+		cmd_list_free(bd->cmdlist);
 		xfree(bd);
 	}
 }
@@ -206,5 +218,5 @@ key_bindings_dispatch(struct key_binding *bd, struct client *c)
 
 	ctx.cmdclient = NULL;
 
-	cmd_exec(bd->cmd, &ctx);
+	cmd_list_exec(bd->cmdlist, &ctx);
 }
