@@ -1,4 +1,4 @@
-/* $Id: paste.c,v 1.4 2008/08/28 17:45:26 nicm Exp $ */
+/* $Id: paste.c,v 1.6 2009/01/25 18:51:28 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -68,7 +68,7 @@ paste_free_top(struct paste_stack *ps)
 	struct paste_buffer	*pb;
 
 	if (ARRAY_LENGTH(ps) == 0)
-		return (1);
+		return (-1);
 
 	pb = ARRAY_FIRST(ps);
 	ARRAY_REMOVE(ps, 0);
@@ -85,7 +85,7 @@ paste_free_index(struct paste_stack *ps, u_int idx)
 	struct paste_buffer	*pb;
 
 	if (idx >= ARRAY_LENGTH(ps))
-		return (1);
+		return (-1);
 
 	pb = ARRAY_ITEM(ps, idx);
 	ARRAY_REMOVE(ps, idx);
@@ -97,7 +97,7 @@ paste_free_index(struct paste_stack *ps, u_int idx)
 }
 
 void
-paste_add(struct paste_stack *ps, const char *data, u_int limit)
+paste_add(struct paste_stack *ps, char *data, u_int limit)
 {
 	struct paste_buffer	*pb;
 
@@ -107,23 +107,23 @@ paste_add(struct paste_stack *ps, const char *data, u_int limit)
 	pb = xmalloc(sizeof *pb);
 	ARRAY_INSERT(ps, 0, pb);
 
-	pb->data = xstrdup(data);
+	pb->data = data;
 	if (gettimeofday(&pb->tv, NULL) != 0)
 		fatal("gettimeofday");
 }
 
 int
-paste_replace(struct paste_stack *ps, u_int idx, const char *data)
+paste_replace(struct paste_stack *ps, u_int idx, char *data)
 {
 	struct paste_buffer	*pb;
 
 	if (idx >= ARRAY_LENGTH(ps))
-		return (1);
+		return (-1);
 
 	pb = ARRAY_ITEM(ps, idx);
 	xfree(pb->data);
 
-	pb->data = xstrdup(data);
+	pb->data = data;
 	if (gettimeofday(&pb->tv, NULL) != 0)
 		fatal("gettimeofday");
 
