@@ -1,4 +1,4 @@
-/* $Id: screen-write.c,v 1.25 2009/01/17 18:47:36 nicm Exp $ */
+/* $Id: screen-write.c,v 1.31 2009/01/28 19:52:21 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -33,13 +33,11 @@ screen_write_start(
 		ctx->data = wp;
 		if (ctx->s == NULL)
 			ctx->s = wp->screen;
+		tty_write_update_mode(ctx->data, ctx->s->mode & ~MODE_CURSOR);
 	} else {
 		ctx->write = NULL;
 		ctx->data = NULL;
 	}
-	
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_CURSORMODE, 0);
 }
 
 /* Finish writing. */
@@ -376,9 +374,6 @@ screen_write_insertmode(struct screen_write_ctx *ctx, int state)
 {
 	struct screen	*s = ctx->s;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_INSERTMODE, state);
-
 	if (state)
 		s->mode |= MODE_INSERT;
 	else
@@ -391,9 +386,6 @@ screen_write_mousemode(struct screen_write_ctx *ctx, int state)
 {
 	struct screen	*s = ctx->s;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_MOUSEMODE, state);
-
 	if (state)
 		s->mode |= MODE_MOUSE;
 	else
@@ -411,7 +403,7 @@ screen_write_linefeed(struct screen_write_ctx *ctx)
 
 	if (s->cy == s->rlower)
 		grid_view_scroll_region_up(s->grid, s->rupper, s->rlower);
-	else if (s->cy < screen_size_x(s) - 1)
+	else if (s->cy < screen_size_y(s) - 1)
 		s->cy++;
 }
 
@@ -430,9 +422,6 @@ screen_write_kcursormode(struct screen_write_ctx *ctx, int state)
 {
 	struct screen	*s = ctx->s;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_KCURSORMODE);
-
 	if (state)
 		s->mode |= MODE_KCURSOR;
 	else
@@ -445,9 +434,6 @@ screen_write_kkeypadmode(struct screen_write_ctx *ctx, int state)
 {
 	struct screen	*s = ctx->s;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_KKEYPADMODE);
-
 	if (state)
 		s->mode |= MODE_KKEYPAD;
 	else
@@ -514,7 +500,15 @@ screen_write_cell(struct screen_write_ctx *ctx, const struct grid_cell *gc)
 	const struct grid_cell 	*hc;
 	struct grid_cell 	*ic, tc;
 
-	width = utf8_width(gc->data);
+	/* Find character width. */
+	if (gc->flags & GRID_FLAG_UTF8)
+		width = utf8_width(gc->data);
+	else
+		width = 1;
+
+	/* Discard zero-width characters. */
+	if (width == 0)
+		return;
 
 	/* If the character is wider than the screen, don't print it. */
 	if (width > screen_size_x(s)) {
@@ -570,7 +564,7 @@ screen_write_cell(struct screen_write_ctx *ctx, const struct grid_cell *gc)
 				break;
 			grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
 		}
-	} else if (utf8_width(hc->data) > 1) {
+	} else if (hc->flags & GRID_FLAG_UTF8 && utf8_width(hc->data) > 1) {
 		/*
 		 * An UTF-8 wide cell; overwrite following padding cells only.
 		 */
