@@ -1,4 +1,4 @@
-/* $Id: server-msg.c,v 1.57 2009/01/11 23:31:46 nicm Exp $ */
+/* $Id: server-msg.c,v 1.62 2009/01/28 17:57:00 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,6 +31,7 @@ int	server_msg_fn_identify(struct hdr *, struct client *);
 int	server_msg_fn_resize(struct hdr *, struct client *);
 int	server_msg_fn_exiting(struct hdr *, struct client *);
 int	server_msg_fn_unlock(struct hdr *, struct client *);
+int	server_msg_fn_wakeup(struct hdr *, struct client *);
 
 void printflike2 server_msg_fn_command_error(
     	    struct cmd_ctx *, const char *, ...);
@@ -48,7 +49,8 @@ const struct server_msg server_msg_table[] = {
 	{ MSG_COMMAND, server_msg_fn_command },
 	{ MSG_RESIZE, server_msg_fn_resize },
 	{ MSG_EXITING, server_msg_fn_exiting },
-	{ MSG_UNLOCK, server_msg_fn_unlock }
+	{ MSG_UNLOCK, server_msg_fn_unlock },
+	{ MSG_WAKEUP, server_msg_fn_wakeup },
 };
 
 int
@@ -130,14 +132,14 @@ server_msg_fn_command(struct hdr *hdr, struct client *c)
 {
 	struct msg_command_data	data;
 	struct cmd_ctx	 	ctx;
+	struct cmd_list	       *cmdlist;
 	struct cmd	       *cmd;
 
 	if (hdr->size < sizeof data)
 		fatalx("bad MSG_COMMAND size");
 	buffer_read(c->in, &data, sizeof data);
 
-	cmd = cmd_recv(c->in);
-	log_debug("got command %s from client %d", cmd->entry->name, c->fd);
+	cmdlist = cmd_list_recv(c->in);
 	server_activity = time(NULL);
 
 	ctx.error = server_msg_fn_command_error;
@@ -150,16 +152,22 @@ server_msg_fn_command(struct hdr *hdr, struct client *c)
 
 	ctx.cmdclient = c;
 
-	/* XXX */
-	if (data.pid != -1 && (cmd->entry->flags & CMD_CANTNEST)) {
-		server_msg_fn_command_error(&ctx, "sessions "
-		    "should be nested with care. unset $TMUX to force");
-		cmd_free(cmd);
-		return (0);
+	if (data.pid != -1) {
+		TAILQ_FOREACH(cmd, cmdlist, qentry) {
+			if (cmd->entry->flags & CMD_CANTNEST) {
+				server_msg_fn_command_error(&ctx, 
+				    "sessions should be nested with care. "
+				    "unset $TMUX to force");
+				cmd_list_free(cmdlist);
+				server_write_client(c, MSG_EXIT, NULL, 0);
+				return (0);
+			}
+		}
 	}
 
-	cmd_exec(cmd, &ctx);
-	cmd_free(cmd);
+	if (cmd_list_exec(cmdlist, &ctx) != 1)
+		server_write_client(c, MSG_EXIT, NULL, 0);
+	cmd_list_free(cmdlist);
 	return (0);
 }
 
@@ -223,7 +231,12 @@ server_msg_fn_resize(struct hdr *hdr, struct client *c)
 	c->sy = data.sy;
 	if (c->sy == 0)
 		c->sy = 25;
-
+	
+	c->tty.cx = UINT_MAX;
+	c->tty.cy = UINT_MAX;
+	c->tty.rupper = UINT_MAX;
+	c->tty.rlower = UINT_MAX;
+	
 	recalculate_sizes();
 
 	/* Always redraw this client. */
@@ -242,7 +255,7 @@ server_msg_fn_exiting(struct hdr *hdr, struct client *c)
 
 	c->session = NULL;
 
-	tty_close(&c->tty);
+	tty_close(&c->tty, c->flags & CLIENT_SUSPENDED);
 
 	server_write_client(c, MSG_EXITED, NULL, 0);
 
@@ -263,6 +276,7 @@ server_msg_fn_unlock(struct hdr *hdr, struct client *c)
 	if (server_unlock(pass) != 0) {
 #define MSG "bad password"
 		server_write_client(c, MSG_ERROR, MSG, (sizeof MSG) - 1);
+		server_write_client(c, MSG_EXIT, NULL, 0);
 		return (0);
 #undef MSG
 	}
@@ -271,3 +285,18 @@ server_msg_fn_unlock(struct hdr *hdr, struct client *c)
 
 	return (0);
 }
+
+int
+server_msg_fn_wakeup(struct hdr *hdr, struct client *c)
+{
+	if (hdr->size != 0)
+		fatalx("bad MSG_WAKEUP size");
+
+	log_debug("wakeup msg from client");
+
+	c->flags &= ~CLIENT_SUSPENDED;
+	tty_start_tty(&c->tty);
+	server_redraw_client(c);
+
+	return (0);
+}
