@@ -1,4 +1,4 @@
-/* $Id: server.c,v 1.106 2009/01/17 17:42:10 nicm Exp $ */
+/* $Id: server.c,v 1.116 2009/01/29 20:13:12 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -43,6 +43,7 @@
 struct clients	 clients;
 
 int		 server_main(const char *, int);
+void		 server_shutdown(void);
 void		 server_fill_windows(struct pollfd **);
 void		 server_handle_windows(struct pollfd **);
 void		 server_fill_clients(struct pollfd **);
@@ -58,6 +59,51 @@ void		 server_check_timers(struct client *);
 void		 server_second_timers(void);
 int		 server_update_socket(const char *);
 
+/* Create a new client. */
+struct client *
+server_create_client(int fd)
+{
+	struct client	*c;
+	int		 mode;
+	u_int		 i;
+
+	if ((mode = fcntl(fd, F_GETFL)) == -1)
+		fatal("fcntl failed");
+	if (fcntl(fd, F_SETFL, mode|O_NONBLOCK) == -1)
+		fatal("fcntl failed");
+
+	c = xcalloc(1, sizeof *c);
+	c->fd = fd;
+	c->in = buffer_create(BUFSIZ);
+	c->out = buffer_create(BUFSIZ);
+
+	ARRAY_INIT(&c->prompt_hdata);
+
+	c->tty.fd = -1;
+	c->title = NULL;
+
+	c->session = NULL;
+	c->sx = 80;
+	c->sy = 25;
+	screen_init(&c->status, c->sx, 1, 0);
+
+	c->message_string = NULL;
+
+	c->prompt_string = NULL;
+	c->prompt_buffer = NULL;
+	c->prompt_index = 0;
+
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		if (ARRAY_ITEM(&clients, i) == NULL) {
+			ARRAY_SET(&clients, i, c);
+			return (c);
+		}
+	}
+	ARRAY_ADD(&clients, c);
+	return (c);
+}
+
+/* Find client index. */
 int
 server_client_index(struct client *c)
 {
@@ -79,9 +125,8 @@ server_start(const char *path)
 	mode_t			mask;
 	int		   	n, fd, pair[2], mode;
 	char		       *cause;
-	u_char			ch;
 
-	/* Make a little socketpair to wait for the server to be ready. */
+	/* The first client is special and gets a socketpair; create it. */
 	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, pair) != 0)
 		fatal("socketpair failed");
 
@@ -92,17 +137,7 @@ server_start(const char *path)
 		break;
 	default:
 		close(pair[1]);
-
-		ch = 0x00;
-		if (read(pair[0], &ch, 1) == 1 && ch == 0xff) {
-			close(pair[0]);
-			return (0);
-		}
-		ch = 0x00;
-		if (write(pair[1], &ch, 1) != 1)
-			fatal("write failed");
-		close(pair[0]);
-		return (1);
+		return (pair[0]);
 	}
 	close(pair[0]);
 
@@ -114,7 +149,7 @@ server_start(const char *path)
 	 * Must daemonise before loading configuration as the PID changes so
 	 * $TMUX would be wrong for sessions created in the config file.
 	 */
-	if (daemon(1, 1) != 0)
+	if (daemon(1, 0) != 0)
 		fatal("daemon failed");
 
 	ARRAY_INIT(&windows);
@@ -166,11 +201,7 @@ server_start(const char *path)
 	if (fcntl(fd, F_SETFD, FD_CLOEXEC) == -1)
 		fatal("fcntl failed");
 
-	ch = 0xff;
-	if (write(pair[1], &ch, 1) != 1)
-		fatal("write failed");
-	read(pair[1], &ch, 1); /* Ignore errors; just to wait before closing. */
-	close(pair[1]);
+	server_create_client(pair[1]);
 
 	n = server_main(path, fd);
 #ifdef DEBUG
@@ -194,7 +225,11 @@ server_main(const char *srv_path, int srv_fd)
 	last = time(NULL);
 
 	pfds = NULL;
-	while (!sigterm) {
+	for (;;) {
+		/* If sigterm, kill all windows and clients. */
+		if (sigterm)
+			server_shutdown();
+
 		/* Initialise pollfd array. */
 		nfds = 1;
 		for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
@@ -217,7 +252,7 @@ server_main(const char *srv_path, int srv_fd)
 
 		/* Update socket permissions. */
 		xtimeout = INFTIM;
-		if (server_update_socket(srv_path) != 0)
+		if (sigterm || server_update_socket(srv_path) != 0)
 			xtimeout = 100;
 
 		/* Do the poll. */
@@ -246,6 +281,9 @@ server_main(const char *srv_path, int srv_fd)
 			server_second_timers();
 		}
 
+		/* Set window names. */
+		set_window_names();
+
 		/*
 		 * Handle window and client sockets. Clients can create
 		 * windows, so windows must come first to avoid messing up by
@@ -293,6 +331,34 @@ server_main(const char *srv_path, int srv_fd)
 	return (0);
 }
 
+/* Kill all clients. */
+void
+server_shutdown(void)
+{
+	struct session	*s;
+	struct client	*c;
+	u_int		 i, j;
+
+	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
+		s = ARRAY_ITEM(&sessions, i);
+		for (j = 0; j < ARRAY_LENGTH(&clients); j++) {
+			c = ARRAY_ITEM(&clients, j);
+			if (c != NULL && c->session == s) {
+				s = NULL;
+				break;
+			}
+		}
+		if (s != NULL)
+			session_destroy(s);
+	}
+
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c != NULL)
+			server_write_client(c, MSG_SHUTDOWN, NULL, 0);
+	}
+}
+	
 /* Fill window pollfds. */
 void
 server_fill_windows(struct pollfd **pfd)
@@ -351,7 +417,7 @@ void
 server_check_redraw(struct client *c)
 {
 	struct session	*s;
-	char		*title;
+	char		 title[512];
 	int		 flags;
 
 	if (c == NULL || c->session == NULL)
@@ -362,7 +428,9 @@ server_check_redraw(struct client *c)
 	c->tty.flags &= ~TTY_FREEZE;
 
 	if (options_get_number(&s->options, "set-titles")) {
-		title = s->curw->window->active->screen->title;
+		xsnprintf(title, sizeof title, "%s:%u:%s - \"%s\"",
+		    s->name, s->curw->idx, s->curw->window->name,
+		    s->curw->window->active->screen->title);
 		if (c->title == NULL || strcmp(title, c->title) != 0) {
 			if (c->title != NULL)
 				xfree(c->title);
@@ -484,7 +552,8 @@ server_fill_clients(struct pollfd **pfd)
 		}
 		(*pfd)++;
 
-		if (c == NULL || c->tty.fd == -1 || c->session == NULL)
+		if (c == NULL || c->flags & CLIENT_SUSPENDED ||
+		    c->tty.fd == -1 || c->session == NULL)
 			(*pfd)->fd = -1;
 		else {
 			(*pfd)->fd = c->tty.fd;
@@ -516,7 +585,8 @@ server_handle_clients(struct pollfd **pfd)
 		}
 		(*pfd)++;
 
-		if (c != NULL && c->tty.fd != -1 && c->session != NULL) {
+		if (c != NULL && !(c->flags & CLIENT_SUSPENDED) &&
+		    c->tty.fd != -1 && c->session != NULL) {
 			if (buffer_poll(*pfd, c->tty.in, c->tty.out) != 0)
 				server_lost_client(c);
 			else
@@ -530,52 +600,21 @@ server_handle_clients(struct pollfd **pfd)
 struct client *
 server_accept_client(int srv_fd)
 {
-	struct client	       *c;
 	struct sockaddr_storage	sa;
 	socklen_t		slen = sizeof sa;
-	int		 	client_fd, mode;
-	u_int			i;
+	int			fd;
 
-	client_fd = accept(srv_fd, (struct sockaddr *) &sa, &slen);
-	if (client_fd == -1) {
+	fd = accept(srv_fd, (struct sockaddr *) &sa, &slen);
+	if (fd == -1) {
 		if (errno == EAGAIN || errno == EINTR || errno == ECONNABORTED)
 			return (NULL);
 		fatal("accept failed");
 	}
-	if ((mode = fcntl(client_fd, F_GETFL)) == -1)
-		fatal("fcntl failed");
-	if (fcntl(client_fd, F_SETFL, mode|O_NONBLOCK) == -1)
-		fatal("fcntl failed");
-
-	c = xcalloc(1, sizeof *c);
-	c->fd = client_fd;
-	c->in = buffer_create(BUFSIZ);
-	c->out = buffer_create(BUFSIZ);
-
-	ARRAY_INIT(&c->prompt_hdata);
-
-	c->tty.fd = -1;
-	c->title = NULL;
-
-	c->session = NULL;
-	c->sx = 80;
-	c->sy = 25;
-	screen_init(&c->status, c->sx, 1, 0);
-
-	c->message_string = NULL;
-
-	c->prompt_string = NULL;
-	c->prompt_buffer = NULL;
-	c->prompt_index = 0;
-
-	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
-		if (ARRAY_ITEM(&clients, i) == NULL) {
-			ARRAY_SET(&clients, i, c);
-			return (c);
-		}
+	if (sigterm) {
+		close(fd);
+		return (NULL);
 	}
-	ARRAY_ADD(&clients, c);
-	return (c);
+	return (server_create_client(fd));
 }
 
 /* Input data from client. */
@@ -585,7 +624,10 @@ server_handle_client(struct client *c)
 	struct window_pane	*wp;
 	struct timeval	 	 tv;
 	struct key_binding	*bd;
-	int		 	 key, prefix, status, flags, xtimeout;
+	struct cmd		*cmd;
+	int		 	 key, prefix, status, xtimeout, can_repeat;
+	int			 mode;
+	u_char			 mouse[3];
 
 	xtimeout = options_get_number(&c->session->options, "repeat-time");
 	if (xtimeout != 0 && c->flags & CLIENT_REPEAT) {
@@ -597,12 +639,12 @@ server_handle_client(struct client *c)
 
 	/* Process keys. */
 	prefix = options_get_number(&c->session->options, "prefix");
-	while (tty_keys_next(&c->tty, &key) == 0) {	
+	while (tty_keys_next(&c->tty, &key, mouse) == 0) {
 		server_activity = time(NULL);
 
 		if (c->session == NULL)
 			return;
-		wp = c->session->curw->window->active;	/* could die - do each loop */
+		wp = c->session->curw->window->active;	/* could die */
 
 		server_clear_client_message(c);
 		if (c->prompt_string != NULL) {
@@ -612,6 +654,12 @@ server_handle_client(struct client *c)
 		if (server_locked)
 			continue;
 		
+		/* Check for mouse keys. */
+		if (key == KEYC_MOUSE) {
+			window_pane_mouse(wp, c, mouse[0], mouse[1], mouse[2]);
+			continue;
+		}
+
 		/* No previous prefix key. */
 		if (!(c->flags & CLIENT_PREFIX)) {
 			if (key == prefix)
@@ -634,10 +682,16 @@ server_handle_client(struct client *c)
 			}
 			continue;
 		}
-		flags = bd->cmd->entry->flags;
+
+		/* Check repeat flag. */
+		can_repeat = 1;
+		TAILQ_FOREACH(cmd, bd->cmdlist, qentry) {
+			if (!(cmd->entry->flags & CMD_CANREPEAT))
+				can_repeat = 0;
+		}
 
 		/* If already repeating, but this key can't repeat, skip it. */
-		if (c->flags & CLIENT_REPEAT && !(flags & CMD_CANREPEAT)) {
+		if (c->flags & CLIENT_REPEAT && !can_repeat) {
 			c->flags &= ~CLIENT_REPEAT;
 			if (key == prefix)
 				c->flags |= CLIENT_PREFIX;
@@ -647,7 +701,7 @@ server_handle_client(struct client *c)
 		}
 		
 		/* If this key can repeat, reset the repeat flags and timer. */
-		if (xtimeout != 0 && flags & CMD_CANREPEAT) {
+		if (xtimeout != 0 && can_repeat) {
 			c->flags |= CLIENT_PREFIX|CLIENT_REPEAT;
 
 			tv.tv_sec = xtimeout / 1000;
@@ -664,15 +718,15 @@ server_handle_client(struct client *c)
 		return;
 	wp = c->session->curw->window->active;	/* could die - do each loop */
 	
-	/* Ensure the cursor is in the right place and correctly on or off. */
+	/* Ensure cursor position and mode settings. */
 	status = options_get_number(&c->session->options, "status");
-	if (c->prompt_string == NULL && c->message_string == NULL &&
-	    !server_locked && wp->screen->mode & MODE_CURSOR &&
-	    wp->yoff + wp->screen->cy < c->sy - status) {
-		tty_write(&c->tty, wp->screen, 0, TTY_CURSORMODE, 1);
+	if (wp->yoff + wp->screen->cy < c->sy - status)
 		tty_cursor(&c->tty, wp->screen->cx, wp->screen->cy, wp->yoff);
-	} else
-		tty_write(&c->tty, wp->screen, 0, TTY_CURSORMODE, 0);
+
+	mode = wp->screen->mode;
+	if (server_locked)
+		mode &= ~TTY_NOCURSOR;
+	tty_update_mode(&c->tty, mode);
 }
 
 /* Lost a client. */
@@ -686,7 +740,7 @@ server_lost_client(struct client *c)
 			ARRAY_SET(&clients, i, NULL);
 	}
 
-	tty_free(&c->tty);
+	tty_free(&c->tty, c->flags & CLIENT_SUSPENDED);
 
 	if (c->title != NULL)
 		xfree(c->title);
@@ -719,7 +773,7 @@ server_handle_window(struct window *w, struct window_pane *wp)
 {
 	struct session	*s;
 	struct client	*c;
-	u_int		 i;
+	u_int		 i, j;
 	int		 action, update;
 
 	window_pane_parse(wp);
@@ -742,8 +796,8 @@ server_handle_window(struct window *w, struct window_pane *wp)
 			case BELL_ANY:
 				if (s->flags & SESSION_UNATTACHED)
 					break;
-				for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
-					c = ARRAY_ITEM(&clients, i);
+				for (j = 0; j < ARRAY_LENGTH(&clients); j++) {
+					c = ARRAY_ITEM(&clients, j);
 					if (c != NULL && c->session == s)
 						tty_putcode(&c->tty, TTYC_BEL);
 				}
@@ -751,8 +805,8 @@ server_handle_window(struct window *w, struct window_pane *wp)
 			case BELL_CURRENT:
 				if (w->active != wp)
 					break;
-				for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
-					c = ARRAY_ITEM(&clients, i);
+				for (j = 0; j < ARRAY_LENGTH(&clients); j++) {
+					c = ARRAY_ITEM(&clients, j);
 					if (c != NULL && c->session == s)
 						tty_putcode(&c->tty, TTYC_BEL);
 				}
