@@ -1,4 +1,4 @@
-/* $Id: session.c,v 1.50 2009/01/12 18:22:47 nicm Exp $ */
+/* $Id: session.c,v 1.52 2009/01/23 16:59:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,6 +28,9 @@
 /* Global session list. */
 struct sessions	sessions;
 
+struct winlink *session_next_activity(struct session *, struct winlink *);
+struct winlink *session_previous_activity(struct session *, struct winlink *);
+
 void
 session_alert_cancel(struct session *s, struct winlink *wl)
 {
@@ -109,8 +112,8 @@ session_find(const char *name)
 
 /* Create a new session. */
 struct session *
-session_create(
-    const char *name, const char *cmd, const char *cwd, u_int sx, u_int sy)
+session_create(const char *name,
+    const char *cmd, const char *cwd, u_int sx, u_int sy, char **cause)
 {
 	struct session	*s;
 	u_int		 i;
@@ -142,7 +145,7 @@ session_create(
 		s->name = xstrdup(name);
 	else
 		xasprintf(&s->name, "%u", i);
-	if (session_new(s, NULL, cmd, cwd, -1) == NULL) {
+	if (session_new(s, NULL, cmd, cwd, -1, cause) == NULL) {
 		session_destroy(s);
 		return (NULL);
 	}
@@ -194,7 +197,7 @@ session_index(struct session *s, u_int *i)
 /* Create a new window on a session. */
 struct winlink *
 session_new(struct session *s,
-    const char *name, const char *cmd, const char *cwd, int idx)
+    const char *name, const char *cmd, const char *cwd, int idx, char **cause)
 {
 	struct window	*w;
 	const char	*env[] = CHILD_ENVIRON;
@@ -207,26 +210,31 @@ session_new(struct session *s,
 	env[0] = buf;
 
 	hlimit = options_get_number(&s->options, "history-limit");
-	w = window_create(name, cmd, cwd, env, s->sx, s->sy, hlimit);
+	w = window_create(name, cmd, cwd, env, s->sx, s->sy, hlimit, cause);
 	if (w == NULL)
 		return (NULL);
 
-	return (session_attach(s, w, idx));
+	return (session_attach(s, w, idx, cause));
 }
 
 /* Attach a window to a session. */
 struct winlink *
-session_attach(struct session *s, struct window *w, int idx)
+session_attach(struct session *s, struct window *w, int idx, char **cause)
 {
-	return (winlink_add(&s->windows, w, idx));
+	struct winlink	*wl;
+
+	if ((wl = winlink_add(&s->windows, w, idx)) == NULL)
+		xasprintf(cause, "index in use: %d", idx);
+	return (wl);
 }
 
 /* Detach a window from a session. */
 int
 session_detach(struct session *s, struct winlink *wl)
 {
-	if (s->curw == wl && session_last(s) != 0 && session_previous(s) != 0)
-		session_next(s);
+	if (s->curw == wl &&
+	    session_last(s) != 0 && session_previous(s, 0) != 0)
+		session_next(s, 0);
 
 	session_alert_cancel(s, wl);
 	winlink_stack_remove(&s->lastw, wl);
@@ -251,9 +259,22 @@ session_has(struct session *s, struct window *w)
 	return (0);
 }
 
+struct winlink *
+session_next_activity(struct session *s, struct winlink *wl)
+{
+	while (wl != NULL) {
+		if (session_alert_has(s, wl, WINDOW_BELL))
+			break;
+		if (session_alert_has(s, wl, WINDOW_ACTIVITY))
+			break;
+		wl = winlink_next(&s->windows, wl);
+	}
+	return (wl);
+}
+
 /* Move session to next window. */
 int
-session_next(struct session *s)
+session_next(struct session *s, int activity)
 {
 	struct winlink	*wl;
 
@@ -261,8 +282,13 @@ session_next(struct session *s)
 		return (-1);
 
 	wl = winlink_next(&s->windows, s->curw);
-	if (wl == NULL)
+	if (activity)
+		wl = session_next_activity(s, wl);
+	if (wl == NULL) {
 		wl = RB_MIN(winlinks, &s->windows);
+		if (activity && ((wl = session_next_activity(s, wl)) == NULL))
+			return (-1);
+	}
 	if (wl == s->curw)
 		return (1);
 	winlink_stack_remove(&s->lastw, wl);
@@ -272,9 +298,22 @@ session_next(struct session *s)
 	return (0);
 }
 
+struct winlink *
+session_previous_activity(struct session *s, struct winlink *wl)
+{
+	while (wl != NULL) {
+		if (session_alert_has(s, wl, WINDOW_BELL))
+			break;
+		if (session_alert_has(s, wl, WINDOW_ACTIVITY))
+			break;
+		wl = winlink_previous(&s->windows, wl);
+	}
+	return (wl);
+}
+
 /* Move session to previous window. */
 int
-session_previous(struct session *s)
+session_previous(struct session *s, int activity)
 {
 	struct winlink	*wl;
 
@@ -282,8 +321,13 @@ session_previous(struct session *s)
 		return (-1);
 
 	wl = winlink_previous(&s->windows, s->curw);
-	if (wl == NULL)
+	if (activity)
+		wl = session_previous_activity(s, wl);
+	if (wl == NULL) {
 		wl = RB_MAX(winlinks, &s->windows);
+		if (activity && (wl = session_previous_activity(s, wl)) == NULL)
+			return (-1);
+	}
 	if (wl == s->curw)
 		return (1);
 	winlink_stack_remove(&s->lastw, wl);
