@@ -1,4 +1,4 @@
-/* $Id: status.c,v 1.64 2009/01/17 18:47:37 nicm Exp $ */
+/* $Id: status.c,v 1.71 2009/02/01 18:14:49 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -30,6 +30,7 @@
 #include "tmux.h"
 
 char   *status_replace(struct session *, char *, time_t);
+char   *status_replace_popen(char **);
 size_t	status_width(struct winlink *);
 char   *status_print(struct session *, struct winlink *, struct grid_cell *);
 
@@ -48,25 +49,34 @@ status_redraw(struct client *c)
 	char		 	       *left, *right, *text, *ptr;
 	size_t				llen, rlen, offset, xx, yy, sy;
 	size_t				size, start, width;
-	struct grid_cell	        gc;
+	struct grid_cell	        stdgc, gc;
 	int				larrow, rarrow;
 
 	left = right = NULL;
 
+	/* Resize the target screen. */
+	if (screen_size_x(&c->status) != c->sx) {
+		screen_free(&c->status);
+		screen_init(&c->status, c->sx, 1, 0);
+	}
+
+	/* No status line? */
 	if (c->sy == 0 || !options_get_number(&s->options, "status"))
 		goto off;
 	larrow = rarrow = 0;
 
 	if (gettimeofday(&c->status_timer, NULL) != 0)
 		fatal("gettimeofday");
-	memcpy(&gc, &grid_default_cell, sizeof gc);
-	gc.fg = options_get_number(&s->options, "status-fg");
-	gc.bg = options_get_number(&s->options, "status-bg");
+	memcpy(&stdgc, &grid_default_cell, sizeof gc);
+	stdgc.bg = options_get_number(&s->options, "status-fg");
+	stdgc.fg = options_get_number(&s->options, "status-bg");
+	stdgc.attr |= options_get_number(&s->options, "status-attr");
 
 	yy = c->sy - 1;
 	if (yy == 0)
 		goto blank;
 
+	/* Work out the left and right strings. */
 	left = status_replace(s, options_get_string(
 	    &s->options, "status-left"), c->status_timer.tv_sec);
 	llen = options_get_number(&s->options, "status-left-length");
@@ -147,12 +157,6 @@ status_redraw(struct client *c)
  	width = xx;
 
 draw:
-	/* Resize the target screen. */
-	if (screen_size_x(&c->status) != c->sx) {
-		screen_free(&c->status);
-		screen_init(&c->status, c->sx, 1, 0);
-	}
-
 	/* Bail here if anything is too small too. XXX. */
 	if (width == 0 || xx == 0)
 		goto blank;
@@ -161,9 +165,9 @@ draw:
 	screen_write_start(&ctx, NULL, &c->status);
 	if (llen != 0) {
  		screen_write_cursormove(&ctx, 0, yy);
-		screen_write_puts(&ctx, &gc, "%s ", left);
+		screen_write_puts(&ctx, &stdgc, "%s ", left);
 		if (larrow)
-			screen_write_putc(&ctx, &gc, ' ');
+			screen_write_putc(&ctx, &stdgc, ' ');
 	} else {
 		if (larrow)
 			screen_write_cursormove(&ctx, 1, yy);
@@ -174,6 +178,7 @@ draw:
 	/* Draw each character in succession. */
 	offset = 0;
 	RB_FOREACH(wl, winlinks, &s->windows) {
+		memcpy(&gc, &stdgc, sizeof gc);
 		text = status_print(s, wl, &gc);
 
 		if (larrow == 1 && offset < start) {
@@ -196,10 +201,9 @@ draw:
 				rarrow = -1;
 		}
 
-		gc.attr &= ~GRID_ATTR_REVERSE;
 		if (offset < start + width) {
 			if (offset >= start) {
-				screen_write_putc(&ctx, &gc, ' ');
+				screen_write_putc(&ctx, &stdgc, ' ');
 			}
 			offset++;
 		}
@@ -209,38 +213,34 @@ draw:
 
 	/* Fill the remaining space if any. */
  	while (offset++ < xx)
-		screen_write_putc(&ctx, &gc, ' ');
+		screen_write_putc(&ctx, &stdgc, ' ');
 
 	/* Draw the last item. */
 	if (rlen != 0) {
 		screen_write_cursormove(&ctx, c->sx - rlen - 1, yy);
-		screen_write_puts(&ctx, &gc, " %s", right);
+		screen_write_puts(&ctx, &stdgc, " %s", right);
 	}
 
 	/* Draw the arrows. */
 	if (larrow != 0) {
+		memcpy(&gc, &stdgc, sizeof gc);
 		if (larrow == -1)
-			gc.attr |= GRID_ATTR_REVERSE;
-		else
-			gc.attr &= ~GRID_ATTR_REVERSE;
+			gc.attr ^= GRID_ATTR_REVERSE;
 		if (llen != 0)
 			screen_write_cursormove(&ctx, llen + 1, yy);
 		else
 			screen_write_cursormove(&ctx, 0, yy);
 		screen_write_putc(&ctx, &gc, '<');
-		gc.attr &= ~GRID_ATTR_REVERSE;
 	}
 	if (rarrow != 0) {
+		memcpy(&gc, &stdgc, sizeof gc);
 		if (rarrow == -1)
-			gc.attr |= GRID_ATTR_REVERSE;
-		else
-			gc.attr &= ~GRID_ATTR_REVERSE;
+			gc.attr ^= GRID_ATTR_REVERSE;
 		if (rlen != 0)
 			screen_write_cursormove(&ctx, c->sx - rlen - 2, yy);
 		else
 			screen_write_cursormove(&ctx, c->sx - 1, yy);
 		screen_write_putc(&ctx, &gc, '>');
-		gc.attr &= ~GRID_ATTR_REVERSE;
 	}
 
 	goto out;
@@ -250,7 +250,7 @@ blank:
 	screen_write_start(&ctx, NULL, &c->status);
 	screen_write_cursormove(&ctx, 0, yy);
 	for (offset = 0; offset < c->sx; offset++)
-		screen_write_putc(&ctx, &gc, ' ');
+		screen_write_putc(&ctx, &stdgc, ' ');
 
 	goto out;
 
@@ -259,20 +259,20 @@ off:
 	 * Draw the real window last line. Necessary to wipe over message if
 	 * status is off. Not sure this is the right place for this.
 	 */
+	memcpy(&stdgc, &grid_default_cell, sizeof stdgc);
 	screen_write_start(&ctx, NULL, &c->status);
 
 	sy = 0;
 	TAILQ_FOREACH(wp, &s->curw->window->panes, entry) {
-		sy += wp->sy;
+		sy += wp->sy + 1;
 		sc = wp->screen;
 	}
-	sy++;
 
 	screen_write_cursormove(&ctx, 0, 0);
-	if (sy < c->sy - 1) {
+	if (sy < c->sy) {
 		/* If the screen is too small, use blank. */
-		for (offset = 0; offset < c->sx; offset++)
-			screen_write_putc(&ctx, &gc, ' ');
+ 		for (offset = 0; offset < c->sx; offset++)
+ 			screen_write_putc(&ctx, &stdgc, ' ');
 	} else {
 		screen_write_copy(&ctx,
 		    sc, 0, sc->grid->hsize + screen_size_y(sc) - 1, c->sx, 1);
@@ -293,6 +293,7 @@ status_replace(struct session *s, char *fmt, time_t t)
 	struct winlink *wl = s->curw;
 	static char	out[BUFSIZ];
 	char		in[BUFSIZ], tmp[256], ch, *iptr, *optr, *ptr, *endptr;
+	char           *savedptr;
 	size_t		len;
 	long		n;
 
@@ -301,6 +302,7 @@ status_replace(struct session *s, char *fmt, time_t t)
 
 	iptr = in;
 	optr = out;
+	savedptr = NULL;
 
 	while (*iptr != '\0') {
 		if (optr >= out + (sizeof out) - 1)
@@ -319,6 +321,14 @@ status_replace(struct session *s, char *fmt, time_t t)
 
 			ptr = NULL;
 			switch (*iptr++) {
+			case '(':
+				if (ptr == NULL) {
+					ptr = status_replace_popen(&iptr);
+					if (ptr == NULL)
+						break;
+					savedptr = ptr;
+				}
+				/* FALLTHROUGH */
 			case 'H':
 				if (ptr == NULL) {
 					if (gethostname(tmp, sizeof tmp) != 0)
@@ -347,6 +357,10 @@ status_replace(struct session *s, char *fmt, time_t t)
 				*optr++ = '#';
 				break;
 			}
+			if (savedptr != NULL) {
+				xfree(savedptr);
+				savedptr = NULL;
+			}
 			break;
 		default:
 			*optr++ = ch;
@@ -358,6 +372,65 @@ status_replace(struct session *s, char *fmt, time_t t)
 	return (xstrdup(out));
 }
 
+char *
+status_replace_popen(char **iptr)
+{
+	FILE	*f;
+	char	*buf, *cmd, *ptr;
+	int	lastesc;
+	size_t	len;
+
+	if (**iptr == '\0')
+		return (NULL);
+	if (**iptr == ')') {		/* no command given */
+		(*iptr)++;
+		return (NULL);
+	}
+
+	buf = NULL;
+
+	cmd = xmalloc(strlen(*iptr) + 1);
+	len = 0;
+
+	lastesc = 0;
+	for (; **iptr != '\0'; (*iptr)++) {
+		if (!lastesc && **iptr == ')')
+			break;		/* unescaped ) is the end */
+		if (!lastesc && **iptr == '\\') {
+			lastesc = 1;
+			continue;	/* skip \ if not escaped */
+		}
+		lastesc = 0;
+		cmd[len++] = **iptr;
+	}
+	if (**iptr == '\0')		/* no terminating ) */
+		goto out;
+	(*iptr)++;			/* skip final ) */
+	cmd[len] = '\0';
+
+	if ((f = popen(cmd, "r")) == NULL)
+		goto out;
+
+	if ((buf = fgetln(f, &len)) == NULL) {
+		pclose(f);
+		goto out;
+	}
+	if (buf[len - 1] == '\n') {
+		buf[len - 1] = '\0';
+		buf = xstrdup(buf);
+	} else {
+		ptr = xmalloc(len + 1);
+		memcpy(ptr, buf, len);
+		ptr[len] = '\0';
+		buf = ptr;
+	}
+	pclose(f);
+
+out:
+	xfree(cmd);
+	return (buf);
+}
+
 size_t
 status_width(struct winlink *wl)
 {
@@ -379,6 +452,17 @@ char *
 status_print(struct session *s, struct winlink *wl, struct grid_cell *gc)
 {
 	char   *text, flag;
+	u_char	fg, bg, attr;
+
+	fg = options_get_number(&wl->window->options, "window-status-fg");
+	if (fg != 8)
+		gc->fg = fg;
+	bg = options_get_number(&wl->window->options, "window-status-bg");
+	if (bg != 8)
+		gc->bg = bg;
+	attr = options_get_number(&wl->window->options, "window-status-attr");
+	if (attr != 0)
+		gc->attr = attr;
 
 	flag = ' ';
  	if (wl == SLIST_FIRST(&s->lastw))
@@ -386,14 +470,13 @@ status_print(struct session *s, struct winlink *wl, struct grid_cell *gc)
 	if (wl == s->curw)
 		flag = '*';
 
-	gc->attr &= ~GRID_ATTR_REVERSE;
 	if (session_alert_has(s, wl, WINDOW_ACTIVITY)) {
 		flag = '#';
-		gc->attr |= GRID_ATTR_REVERSE;
+		gc->attr ^= GRID_ATTR_REVERSE;
 	}
 	if (session_alert_has(s, wl, WINDOW_BELL)) {
 		flag = '!';
-		gc->attr |= GRID_ATTR_REVERSE;
+		gc->attr ^= GRID_ATTR_REVERSE;
 	}
 
 	xasprintf(&text, "%d:%s%c", wl->idx, wl->window->name, flag);
@@ -421,8 +504,9 @@ status_message_redraw(struct client *c)
 		len = c->sx;
 
 	memcpy(&gc, &grid_default_cell, sizeof gc);
-	gc.fg = options_get_number(&s->options, "message-fg");
-	gc.bg = options_get_number(&s->options, "message-bg");
+	gc.bg = options_get_number(&s->options, "message-fg");
+	gc.fg = options_get_number(&s->options, "message-bg");
+	gc.attr |= options_get_number(&s->options, "message-attr");
 
 	screen_write_start(&ctx, NULL, &c->status);
 
@@ -457,8 +541,9 @@ status_prompt_redraw(struct client *c)
 		len = c->sx;
 
 	memcpy(&gc, &grid_default_cell, sizeof gc);
-	gc.fg = options_get_number(&s->options, "message-fg");
-	gc.bg = options_get_number(&s->options, "message-bg");
+	gc.bg = options_get_number(&s->options, "message-fg");
+	gc.fg = options_get_number(&s->options, "message-bg");
+	gc.attr |= options_get_number(&s->options, "message-attr");
 
 	screen_write_start(&ctx, NULL, &c->status);
 
@@ -498,8 +583,7 @@ status_prompt_redraw(struct client *c)
 		ch = c->prompt_buffer[c->prompt_index];
 	if (ch == '\0')
 		ch = ' ';
-	gc.bg = gc.fg;
-	gc.fg = options_get_number(&s->options, "message-bg");
+	gc.attr ^= GRID_ATTR_REVERSE;
 	screen_write_putc(&ctx, &gc, ch);
 
 	screen_write_stop(&ctx);
