@@ -1,4 +1,4 @@
-/* $Id: tmux.c,v 1.97 2009/01/15 23:42:21 nicm Exp $ */
+/* $Id: tmux.c,v 1.104 2009/01/30 00:24:49 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -45,6 +45,7 @@ const char	*_malloc_options = "AJX";
 
 volatile sig_atomic_t sigwinch;
 volatile sig_atomic_t sigterm;
+volatile sig_atomic_t sigcont;
 
 char		*cfg_file;
 struct options	 global_options;
@@ -59,7 +60,6 @@ int		 be_quiet;
 time_t		 start_time;
 const char	*socket_path;
 
-void		 sighandler(int);
 __dead void	 usage(void);
 
 #ifdef NO_PROGNAME
@@ -105,6 +105,9 @@ sighandler(int sig)
 	case SIGCHLD:
 		waitpid(WAIT_ANY, NULL, WNOHANG);
 		break;
+	case SIGCONT:
+		sigcont = 1;
+		break;
 	}
 	errno = saved_errno;
 }
@@ -176,14 +179,15 @@ main(int argc, char **argv)
 	struct client_ctx	 cctx;
 	struct msg_command_data	 cmddata;
 	struct buffer		*b;
-	struct cmd		*cmd;
+	struct cmd_list		*cmdlist;
+ 	struct cmd		*cmd;
 	struct pollfd	 	 pfd;
 	struct hdr	 	 hdr;
 	const char		*shell;
 	struct passwd		*pw;
 	char			*path, *cause, *home, *pass = NULL;
 	char			 rpath[MAXPATHLEN], cwd[MAXPATHLEN];
-	int	 		 n, opt, flags, unlock, start_server;
+	int	 		 retcode, opt, flags, unlock, start_server;
 
 	unlock = flags = 0;
 	path = NULL;
@@ -233,6 +237,7 @@ main(int argc, char **argv)
 	options_set_number(&global_options, "history-limit", 2000);
 	options_set_number(&global_options, "message-bg", 3);
 	options_set_number(&global_options, "message-fg", 0);
+	options_set_number(&global_options, "message-attr", GRID_ATTR_REVERSE);
 	options_set_number(&global_options, "prefix", META);
 	options_set_number(&global_options, "repeat-time", 500);
 	options_set_number(&global_options, "set-titles", 1);
@@ -240,6 +245,7 @@ main(int argc, char **argv)
 	options_set_number(&global_options, "status", 1);
 	options_set_number(&global_options, "status-bg", 2);
 	options_set_number(&global_options, "status-fg", 0);
+	options_set_number(&global_options, "status-attr", GRID_ATTR_REVERSE);
 	options_set_number(&global_options, "status-interval", 15);
 	options_set_number(&global_options, "status-left-length", 10);
 	options_set_number(&global_options, "status-right-length", 40);
@@ -252,13 +258,19 @@ main(int argc, char **argv)
 	options_set_number(&global_window_options, "clock-mode-style", 1);
 	options_set_number(&global_window_options, "force-height", 0);
 	options_set_number(&global_window_options, "force-width", 0);
+	options_set_number(&global_window_options, "automatic-rename", 1);
 	options_set_number(&global_window_options, "mode-bg", 3);
 	options_set_number(&global_window_options, "mode-fg", 0);
+	options_set_number(
+	    &global_window_options, "mode-attr", GRID_ATTR_REVERSE);
 	options_set_number(&global_window_options, "mode-keys", MODEKEY_EMACS);
 	options_set_number(&global_window_options, "monitor-activity", 0);
 	options_set_number(&global_window_options, "utf8", 0);
 	options_set_number(&global_window_options, "xterm-keys", 0);
  	options_set_number(&global_window_options, "remain-on-exit", 0);
+	options_set_number(&global_window_options, "window-status-bg", 8);
+	options_set_number(&global_window_options, "window-status-fg", 8);
+	options_set_number(&global_window_options, "window-status-attr", 0);
 
 	if (cfg_file == NULL) {
 		home = getenv("HOME");
@@ -325,7 +337,7 @@ main(int argc, char **argv)
 			log_warnx("can't specify a command when unlocking");
 			exit(1);
 		}
-		cmd = NULL;
+		cmdlist = NULL;
 		if ((pass = getpass("Password: ")) == NULL)
 			exit(1);
 		start_server = 0;
@@ -334,11 +346,24 @@ main(int argc, char **argv)
 			cmd = xmalloc(sizeof *cmd);
 			cmd->entry = &cmd_new_session_entry;
 			cmd->entry->init(cmd, 0);
-		} else if ((cmd = cmd_parse(argc, argv, &cause)) == NULL) {
-			log_warnx("%s", cause);
-			exit(1);
+
+			cmdlist = xmalloc(sizeof *cmdlist);
+			TAILQ_INIT(cmdlist);
+			TAILQ_INSERT_HEAD(cmdlist, cmd, qentry);
+		} else {
+			cmdlist = cmd_list_parse(argc, argv, &cause);
+			if (cmdlist == NULL) {
+				log_warnx("%s", cause);
+				exit(1);
+			}
+		}
+		start_server = 0;
+		TAILQ_FOREACH(cmd, cmdlist, qentry) {
+			if (cmd->entry->flags & CMD_STARTSERVER) {
+				start_server = 1;
+				break;
+			}
 		}
-		start_server = cmd->entry->flags & CMD_STARTSERVER;
 	}
 	
  	memset(&cctx, 0, sizeof cctx);
@@ -351,14 +376,15 @@ main(int argc, char **argv)
 		client_write_server(
 		    &cctx, MSG_UNLOCK, BUFFER_OUT(b), BUFFER_USED(b));
 	} else {
-		cmd_send(cmd, b);
-		cmd_free(cmd);
+		cmd_list_send(cmdlist, b);
+		cmd_list_free(cmdlist);
 		client_fill_session(&cmddata);
 		client_write_server2(&cctx, MSG_COMMAND,
 		    &cmddata, sizeof cmddata, BUFFER_OUT(b), BUFFER_USED(b));
 	}
 	buffer_destroy(b);
 
+	retcode = 0;
 	for (;;) {
 		pfd.fd = cctx.srv_fd;
 		pfd.events = POLLIN;
@@ -384,8 +410,11 @@ main(int argc, char **argv)
 
 		switch (hdr.type) {
 		case MSG_EXIT:
-			n = 0;
+		case MSG_SHUTDOWN:
 			goto out;
+		case MSG_ERROR:
+			retcode = 1;
+			/* FALLTHROUGH */
 		case MSG_PRINT:
 			if (hdr.size > INT_MAX - 1)
 				fatalx("bad MSG_PRINT size");
@@ -394,17 +423,8 @@ main(int argc, char **argv)
 			if (hdr.size != 0)
 				buffer_remove(cctx.srv_in, hdr.size);
 			goto restart;
-		case MSG_ERROR:
-			if (hdr.size > INT_MAX - 1)
-				fatalx("bad MSG_ERROR size");
-			log_warnx("%.*s",
-			    (int) hdr.size, BUFFER_OUT(cctx.srv_in));
-			if (hdr.size != 0)
-				buffer_remove(cctx.srv_in, hdr.size);
-			n = 1;
-			goto out;
 		case MSG_READY:
-			n = client_main(&cctx);
+			retcode = client_main(&cctx);
 			goto out;
 		default:
 			fatalx("unexpected command");
@@ -422,5 +442,5 @@ out:
 #ifdef DEBUG
 	xmalloc_report(getpid(), "client");
 #endif
-	return (n);
+	return (retcode);
 }
