@@ -1,4 +1,4 @@
-/* $Id: tmux.h,v 1.236 2009/01/17 18:38:12 nicm Exp $ */
+/* $Id: tmux.h,v 1.259 2009/02/03 17:21:19 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,7 +19,7 @@
 #ifndef TMUX_H
 #define TMUX_H
 
-#define PROTOCOL_VERSION -8
+#define PROTOCOL_VERSION -11
 
 /* Shut up gcc warnings about empty if bodies. */
 #define RB_AUGMENT(x) do {} while (0)
@@ -123,6 +123,9 @@ extern const char    *__progname;
 /* Minimum pane size. */
 #define PANE_MINIMUM 4	/* includes separator line */
 
+/* Automatic name refresh interval, in milliseconds. */
+#define NAME_INTERVAL 100
+
 /* Fatal errors. */
 #define fatal(msg) log_fatal("%s: %s", __func__, msg);
 #define fatalx(msg) log_fatalx("%s: %s", __func__, msg);
@@ -137,7 +140,7 @@ extern const char    *__progname;
 #define printflike4 __attribute__ ((format (printf, 4, 5)))
 
 /* Number of items in array. */
-#define nitems(_a)  (sizeof((_a)) / sizeof((_a)[0]))
+#define nitems(_a) (sizeof((_a)) / sizeof((_a)[0]))
 
 /* Buffer macros. */
 #define BUFFER_USED(b) ((b)->size)
@@ -178,6 +181,9 @@ struct buffer {
 #define KEYC_REMOVESFT(k) ((k) & ~KEYC_SHIFT)
 #define KEYC_ISSFT(k) ((k) != KEYC_NONE && ((k) & KEYC_SHIFT))
 
+/* Mouse key. */
+#define KEYC_MOUSE (KEYC_OFFSET + 0x00)
+
 /* Function keys. */
 #define KEYC_F1 (KEYC_OFFSET + 0x01)
 #define KEYC_F2 (KEYC_OFFSET + 0x02)
@@ -332,16 +338,11 @@ enum tty_cmd {
 	TTY_CLEARSCREEN,
 	TTY_CLEARSTARTOFLINE,
 	TTY_CLEARSTARTOFSCREEN,
-	TTY_CURSORMODE,
 	TTY_DELETECHARACTER,
 	TTY_DELETELINE,
 	TTY_INSERTCHARACTER,
 	TTY_INSERTLINE,
-	TTY_INSERTMODE,
-	TTY_KCURSORMODE,
-	TTY_KKEYPADMODE,
 	TTY_LINEFEED,
-	TTY_MOUSEMODE,
 	TTY_REVERSEINDEX,
 };
 
@@ -357,7 +358,10 @@ enum hdrtype {
 	MSG_PRINT,
 	MSG_READY,
 	MSG_RESIZE,
+	MSG_SHUTDOWN,
+	MSG_SUSPEND,
 	MSG_UNLOCK,
+	MSG_WAKEUP,
 };
 
 /* Message header structure. */
@@ -424,6 +428,7 @@ struct msg_resize_data {
 #define GRID_FLAG_FG256 0x1
 #define GRID_FLAG_BG256 0x2
 #define GRID_FLAG_PADDING 0x4
+#define GRID_FLAG_UTF8 0x8
 
 /* Grid cell. */
 struct grid_cell {
@@ -454,13 +459,11 @@ struct options_entry {
 		OPTIONS_STRING,
 		OPTIONS_NUMBER,
 		OPTIONS_KEY,
-		OPTIONS_COLOURS
 	} type;
 	union {
 		char	*string;
 		long long number;
 		int	 key;
-		u_char	 colours;
 	} value;
 
 	SPLAY_ENTRY(options_entry) entry;
@@ -566,6 +569,8 @@ struct window_mode {
 	void	(*free)(struct window_pane *);
 	void	(*resize)(struct window_pane *, u_int, u_int);
 	void	(*key)(struct window_pane *, struct client *, int);
+	void	(*mouse)(struct window_pane *,
+	    	    struct client *, u_char, u_char, u_char);
 	void	(*timer)(struct window_pane *);
 };
 
@@ -585,6 +590,7 @@ struct window_pane {
 	char		*cwd;
 
 	int		 fd;
+	char		 tty[TTY_NAME_MAX];
 	struct buffer	*in;
 	struct buffer	*out;
 
@@ -603,6 +609,8 @@ TAILQ_HEAD(window_panes, window_pane);
 /* Window structure. */
 struct window {
 	char		*name;
+	struct timeval	 name_timer;
+	pid_t		 pgrp;
 
 	struct window_pane *active;	
 	struct window_panes panes;
@@ -701,7 +709,7 @@ struct tty {
 	u_int		 cx;
 	u_int		 cy;
 
-	int		 cursor;
+	int		 mode;
 
 	u_int		 rlower;
 	u_int		 rupper;
@@ -725,7 +733,7 @@ struct tty {
 #define TTY_FREEZE 0x2
 #define TTY_ESCAPE 0x4
 #define TTY_UTF8 0x8
-	int		 flags;
+	int    		 flags;
 
 	int		 term_flags;
 
@@ -759,6 +767,7 @@ struct client {
 #define CLIENT_REDRAW 0x8
 #define CLIENT_STATUS 0x10
 #define CLIENT_REPEAT 0x20	/* allow command to repeat within repeat time */
+#define CLIENT_SUSPENDED 0x40
 	int		 flags;
 
 	char		*message_string;
@@ -785,6 +794,7 @@ struct client_ctx {
 
 #define CCTX_DETACH 0x1
 #define CCTX_EXIT 0x2
+#define CCTX_SHUTDOWN 0x4
 	int 		 flags;
 };
 
@@ -804,7 +814,10 @@ struct cmd_ctx {
 struct cmd {
 	const struct cmd_entry *entry;
 	void	       	*data;
+
+	TAILQ_ENTRY(cmd) qentry;
 };
+TAILQ_HEAD(cmd_list, cmd);
 
 struct cmd_entry {
 	const char	*name;
@@ -825,11 +838,11 @@ struct cmd_entry {
 
 	void		 (*init)(struct cmd *, int);
 	int		 (*parse)(struct cmd *, int, char **, char **);
-	void		 (*exec)(struct cmd *, struct cmd_ctx *);
+	int		 (*exec)(struct cmd *, struct cmd_ctx *);
 	void		 (*send)(struct cmd *, struct buffer *);
 	void	         (*recv)(struct cmd *, struct buffer *);
 	void		 (*free)(struct cmd *);
-	void 		 (*print)(struct cmd *, char *, size_t);
+	size_t 		 (*print)(struct cmd *, char *, size_t);
 };
 
 /* Generic command data. */
@@ -870,7 +883,7 @@ struct cmd_pane_data {
 /* Key binding. */
 struct key_binding {
 	int		 key;
-	struct cmd	*cmd;
+	struct cmd_list	*cmdlist;
 
 	SPLAY_ENTRY(key_binding) entry;
 };
@@ -884,6 +897,7 @@ struct set_option_entry {
 		SET_OPTION_NUMBER,
 		SET_OPTION_KEY,
 		SET_OPTION_COLOUR,
+		SET_OPTION_ATTRIBUTES,
 		SET_OPTION_FLAG,
 		SET_OPTION_CHOICE
 	} type;
@@ -895,8 +909,8 @@ struct set_option_entry {
 };
 extern const struct set_option_entry set_option_table[];
 extern const struct set_option_entry set_window_option_table[];
-#define NSETOPTION 20
-#define NSETWINDOWOPTION 12
+#define NSETOPTION 22
+#define NSETWINDOWOPTION 17
 
 /* Edit keys. */
 enum mode_key {
@@ -959,6 +973,7 @@ char   *fgetln(FILE *, size_t *);
 /* tmux.c */
 extern volatile sig_atomic_t sigwinch;
 extern volatile sig_atomic_t sigterm;
+extern volatile sig_atomic_t sigcont;
 extern struct options global_options;
 extern struct options global_window_options;
 extern char	*cfg_file;
@@ -972,6 +987,7 @@ extern const char *socket_path;
 void		 logfile(const char *);
 void		 siginit(void);
 void		 sigreset(void);
+void		 sighandler(int);
 
 /* cfg.c */
 int		 load_cfg(const char *, char **x);
@@ -1001,14 +1017,17 @@ void		 tty_putcode2(struct tty *, enum tty_code_code, int, int);
 void		 tty_puts(struct tty *, const char *);
 void		 tty_putc(struct tty *, char);
 void		 tty_init(struct tty *, char *, char *);
+void		 tty_start_tty(struct tty *);
+void		 tty_stop_tty(struct tty *);
 void		 tty_set_title(struct tty *, const char *);
+void		 tty_update_mode(struct tty *, int);
 int		 tty_open(struct tty *, char **);
-void		 tty_close(struct tty *);
-void		 tty_free(struct tty *);
+void		 tty_close(struct tty *, int);
+void		 tty_free(struct tty *, int);
 void		 tty_write(struct tty *,
 		     struct screen *, u_int, enum tty_cmd, ...);
 void		 tty_vwrite(struct tty *,
-		     struct screen *s, u_int, enum tty_cmd, va_list);
+		     struct screen *, u_int, enum tty_cmd, va_list);
 
 /* tty-term.c */
 extern struct tty_terms tty_terms;
@@ -1028,11 +1047,12 @@ int		 tty_keys_cmp(struct tty_key *, struct tty_key *);
 RB_PROTOTYPE(tty_keys, tty_key, entry, tty_keys_cmp);
 void		 tty_keys_init(struct tty *);
 void		 tty_keys_free(struct tty *);
-int		 tty_keys_next(struct tty *, int *);
+int		 tty_keys_next(struct tty *, int *, u_char *);
 
 /* tty-write.c */
 void		 tty_write_window(void *, enum tty_cmd, ...);
 void		 tty_vwrite_window(void *, enum tty_cmd, va_list);
+void		 tty_write_update_mode(void *, int);
 
 /* options-cmd.c */
 void	set_option_string(struct cmd_ctx *,
@@ -1043,6 +1063,8 @@ void	set_option_key(struct cmd_ctx *,
     	    struct options *, const struct set_option_entry *, char *);
 void	set_option_colour(struct cmd_ctx *,
     	    struct options *, const struct set_option_entry *, char *);
+void	set_option_attributes(struct cmd_ctx *,
+    	    struct options *, const struct set_option_entry *, char *);
 void	set_option_flag(struct cmd_ctx *,
     	    struct options *, const struct set_option_entry *, char *);
 void	set_option_choice(struct cmd_ctx *,
@@ -1056,8 +1078,8 @@ struct paste_buffer *paste_get_top(struct paste_stack *);
 struct paste_buffer *paste_get_index(struct paste_stack *, u_int);
 int	     	 paste_free_top(struct paste_stack *);
 int		 paste_free_index(struct paste_stack *, u_int);
-void		 paste_add(struct paste_stack *, const char *, u_int);
-int		 paste_replace(struct paste_stack *, u_int, const char *);
+void		 paste_add(struct paste_stack *, char *, u_int);
+int		 paste_replace(struct paste_stack *, u_int, char *);
 
 /* clock.c */
 void		 clock_draw(struct screen_write_ctx *, u_int, int);
@@ -1069,10 +1091,11 @@ int		 arg_parse_window(const char *, struct session **, int *);
 
 /* cmd.c */
 struct cmd	*cmd_parse(int, char **, char **);
-void		 cmd_exec(struct cmd *, struct cmd_ctx *);
+int		 cmd_exec(struct cmd *, struct cmd_ctx *);
 void		 cmd_send(struct cmd *, struct buffer *);
 struct cmd	*cmd_recv(struct buffer *);
 void		 cmd_free(struct cmd *);
+size_t		 cmd_print(struct cmd *, char *, size_t);
 void		 cmd_send_string(struct buffer *, const char *);
 char		*cmd_recv_string(struct buffer *);
 struct session	*cmd_current_session(struct cmd_ctx *);
@@ -1087,10 +1110,12 @@ extern const struct cmd_entry cmd_choose_session_entry;
 extern const struct cmd_entry cmd_choose_window_entry;
 extern const struct cmd_entry cmd_clock_mode_entry;
 extern const struct cmd_entry cmd_command_prompt_entry;
+extern const struct cmd_entry cmd_copy_buffer_entry;
 extern const struct cmd_entry cmd_copy_mode_entry;
 extern const struct cmd_entry cmd_delete_buffer_entry;
 extern const struct cmd_entry cmd_detach_client_entry;
 extern const struct cmd_entry cmd_down_pane_entry;
+extern const struct cmd_entry cmd_find_window_entry;
 extern const struct cmd_entry cmd_has_session_entry;
 extern const struct cmd_entry cmd_kill_pane_entry;
 extern const struct cmd_entry cmd_kill_server_entry;
@@ -1104,6 +1129,7 @@ extern const struct cmd_entry cmd_list_commands_entry;
 extern const struct cmd_entry cmd_list_keys_entry;
 extern const struct cmd_entry cmd_list_sessions_entry;
 extern const struct cmd_entry cmd_list_windows_entry;
+extern const struct cmd_entry cmd_load_buffer_entry;
 extern const struct cmd_entry cmd_lock_server_entry;
 extern const struct cmd_entry cmd_move_window_entry;
 extern const struct cmd_entry cmd_new_session_entry;
@@ -1135,66 +1161,71 @@ extern const struct cmd_entry cmd_show_window_options_entry;
 extern const struct cmd_entry cmd_source_file_entry;
 extern const struct cmd_entry cmd_split_window_entry;
 extern const struct cmd_entry cmd_start_server_entry;
+extern const struct cmd_entry cmd_suspend_client_entry;
 extern const struct cmd_entry cmd_swap_window_entry;
 extern const struct cmd_entry cmd_switch_client_entry;
 extern const struct cmd_entry cmd_unbind_key_entry;
 extern const struct cmd_entry cmd_unlink_window_entry;
 extern const struct cmd_entry cmd_up_pane_entry;
 
+/* cmd-list.c */
+struct cmd_list	*cmd_list_parse(int, char **, char **);
+int		 cmd_list_exec(struct cmd_list *, struct cmd_ctx *);
+void		 cmd_list_send(struct cmd_list *, struct buffer *);
+struct cmd_list	*cmd_list_recv(struct buffer *);
+void		 cmd_list_free(struct cmd_list *);
+size_t		 cmd_list_print(struct cmd_list *, char *, size_t);
+
 /* cmd-string.c */
-int	cmd_string_parse(const char *, struct cmd **, char **);
+int	cmd_string_parse(const char *, struct cmd_list **, char **);
 
 /* cmd-generic.c */
+size_t  cmd_prarg(char *, size_t, const char *, char *);
 #define CMD_TARGET_WINDOW_USAGE "[-t target-window]"
 #define CMD_TARGET_SESSION_USAGE "[-t target-session]"
 #define CMD_TARGET_CLIENT_USAGE "[-t target-client]"
 void	cmd_target_init(struct cmd *, int);
 int	cmd_target_parse(struct cmd *, int, char **, char **);
-void	cmd_target_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_target_send(struct cmd *, struct buffer *);
 void	cmd_target_recv(struct cmd *, struct buffer *);
 void	cmd_target_free(struct cmd *);
-void	cmd_target_print(struct cmd *, char *, size_t);
+size_t	cmd_target_print(struct cmd *, char *, size_t);
 #define CMD_SRCDST_WINDOW_USAGE "[-s src-window] [-t dst-window]"
 #define CMD_SRCDST_SESSION_USAGE "[-s src-session] [-t dst-session]"
 #define CMD_SRCDST_CLIENT_USAGE "[-s src-client] [-t dst-client]"
 void	cmd_srcdst_init(struct cmd *, int);
 int	cmd_srcdst_parse(struct cmd *, int, char **, char **);
-void	cmd_srcdst_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_srcdst_send(struct cmd *, struct buffer *);
 void	cmd_srcdst_recv(struct cmd *, struct buffer *);
 void	cmd_srcdst_free(struct cmd *);
-void	cmd_srcdst_print(struct cmd *, char *, size_t);
+size_t	cmd_srcdst_print(struct cmd *, char *, size_t);
 #define CMD_BUFFER_WINDOW_USAGE "[-b buffer-index] [-t target-window]"
 #define CMD_BUFFER_SESSION_USAGE "[-b buffer-index] [-t target-session]"
 #define CMD_BUFFER_CLIENT_USAGE "[-b buffer-index] [-t target-client]"
 void	cmd_buffer_init(struct cmd *, int);
 int	cmd_buffer_parse(struct cmd *, int, char **, char **);
-void	cmd_buffer_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_buffer_send(struct cmd *, struct buffer *);
 void	cmd_buffer_recv(struct cmd *, struct buffer *);
 void	cmd_buffer_free(struct cmd *);
-void	cmd_buffer_print(struct cmd *, char *, size_t);
+size_t	cmd_buffer_print(struct cmd *, char *, size_t);
 #define CMD_OPTION_WINDOW_USAGE "[-gu] [-t target-window] option [value]"
 #define CMD_OPTION_SESSION_USAGE "[-gu] [-t target-session] option [value]"
 #define CMD_OPTION_CLIENT_USAGE "[-gu] [-t target-client] option [value]"
 void	cmd_option_init(struct cmd *, int);
 int	cmd_option_parse(struct cmd *, int, char **, char **);
-void	cmd_option_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_option_send(struct cmd *, struct buffer *);
 void	cmd_option_recv(struct cmd *, struct buffer *);
 void	cmd_option_free(struct cmd *);
-void	cmd_option_print(struct cmd *, char *, size_t);
+size_t	cmd_option_print(struct cmd *, char *, size_t);
 #define CMD_PANE_WINDOW_USAGE "[-t target-window] [-p pane-index]"
 #define CMD_PANE_SESSION_USAGE "[-t target-session] [-p pane-index]"
 #define CMD_PANE_CLIENT_USAGE "[-t target-client] [-p pane-index]"
 void	cmd_pane_init(struct cmd *, int);
 int	cmd_pane_parse(struct cmd *, int, char **, char **);
-void	cmd_pane_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_pane_send(struct cmd *, struct buffer *);
 void	cmd_pane_recv(struct cmd *, struct buffer *);
 void	cmd_pane_free(struct cmd *);
-void	cmd_pane_print(struct cmd *, char *, size_t);
+size_t	cmd_pane_print(struct cmd *, char *, size_t);
 
 /* client.c */
 int	 client_init(const char *, struct client_ctx *, int, int);
@@ -1215,7 +1246,7 @@ extern struct key_bindings key_bindings;
 int	 key_bindings_cmp(struct key_binding *, struct key_binding *);
 SPLAY_PROTOTYPE(key_bindings, key_binding, entry, key_bindings_cmp);
 struct key_binding *key_bindings_lookup(int);
-void	 key_bindings_add(int, struct cmd *);
+void	 key_bindings_add(int, struct cmd_list *);
 void	 key_bindings_remove(int);
 void	 key_bindings_init(void);
 void	 key_bindings_free(void);
@@ -1230,6 +1261,7 @@ const char *key_string_lookup_key(int);
 
 /* server.c */
 extern struct clients clients;
+struct client *server_create_client(int);
 int	 server_client_index(struct client *);
 int	 server_start(const char *);
 
@@ -1276,12 +1308,17 @@ void	 input_parse(struct window_pane *);
 
 /* input-key.c */
 void	 input_key(struct window_pane *, int);
+void	 input_mouse(struct window_pane *, u_char, u_char, u_char);
 
 /* colour.c */
 const char *colour_tostring(u_char);
-u_char	 colour_fromstring(const char *);
+int	 colour_fromstring(const char *);
 u_char	 colour_translate256(u_char);
 
+/* attributes.c */
+const char *attributes_tostring(u_char);
+int	 attributes_fromstring(const char *);
+
 /* grid.c */
 extern const struct grid_cell grid_default_cell;
 struct grid_data *grid_create(u_int, u_int, u_int);
@@ -1393,12 +1430,12 @@ void		 winlink_stack_push(struct winlink_stack *, struct winlink *);
 void		 winlink_stack_remove(struct winlink_stack *, struct winlink *);
 int	 	 window_index(struct window *, u_int *);
 struct window	*window_create(const char *, const char *,
-		     const char *, const char **, u_int, u_int, u_int);
+		     const char *, const char **, u_int, u_int, u_int, char **);
 void		 window_destroy(struct window *);
 int		 window_resize(struct window *, u_int, u_int);
 void		 window_set_active_pane(struct window *, struct window_pane *);
-struct window_pane *window_add_pane(struct window *,
-    		     const char *, const char *, const char **, u_int);
+struct window_pane *window_add_pane(struct window *, int,
+		     const char *, const char *, const char **, u_int, char **);
 void		 window_remove_pane(struct window *, struct window_pane *);
 u_int		 window_index_of_pane(struct window *, struct window_pane *);
 struct window_pane *window_pane_at_index(struct window *, u_int);
@@ -1409,7 +1446,7 @@ void		 window_destroy_panes(struct window *);
 struct window_pane *window_pane_create(struct window *, u_int, u_int, u_int);
 void		 window_pane_destroy(struct window_pane *);
 int		 window_pane_spawn(struct window_pane *,
-		     const char *, const char *, const char **);
+		     const char *, const char *, const char **, char **);
 int		 window_pane_resize(struct window_pane *, u_int, u_int);
 void		 window_calculate_sizes(struct window *);
 int		 window_pane_set_mode(
@@ -1417,15 +1454,19 @@ int		 window_pane_set_mode(
 void		 window_pane_reset_mode(struct window_pane *);
 void		 window_pane_parse(struct window_pane *);
 void		 window_pane_key(struct window_pane *, struct client *, int);
+void		 window_pane_mouse(struct window_pane *,
+    		     struct client *, u_char, u_char, u_char);
 
 /* window-clock.c */
 extern const struct window_mode window_clock_mode;
 
 /* window-copy.c */
 extern const struct window_mode window_copy_mode;
+void 		 window_copy_pageup(struct window_pane *);
 
 /* window-scroll.c */
 extern const struct window_mode window_scroll_mode;
+void 		 window_scroll_pageup(struct window_pane *);
 
 /* window-more.c */
 extern const struct window_mode window_more_mode;
@@ -1441,6 +1482,10 @@ void printflike3 window_choose_add(
 void		 window_choose_ready(struct window_pane *,
 		     u_int, void (*)(void *, int), void *);
 
+/* names.c */
+void		 set_window_names(void);
+char 		*default_window_name(struct window *);
+
 /* session.c */
 extern struct sessions sessions;
 void	 session_alert_add(struct session *, struct window *, int);
@@ -1448,17 +1493,18 @@ void	 session_alert_cancel(struct session *, struct winlink *);
 int	 session_alert_has(struct session *, struct winlink *, int);
 int	 session_alert_has_window(struct session *, struct window *, int);
 struct session	*session_find(const char *);
-struct session	*session_create(
-    		     const char *, const char *, const char *, u_int, u_int);
+struct session	*session_create(const char *, const char *,
+    		     const char *, u_int, u_int, char **);
 void	 	 session_destroy(struct session *);
 int	 	 session_index(struct session *, u_int *);
 struct winlink	*session_new(struct session *, 
-		     const char *, const char *, const char *, int);
-struct winlink	*session_attach(struct session *, struct window *, int);
+	    	     const char *, const char *, const char *, int, char **);
+struct winlink	*session_attach(
+    		     struct session *, struct window *, int, char **);
 int		 session_detach(struct session *, struct winlink *);
 int		 session_has(struct session *, struct window *);
-int		 session_next(struct session *);
-int		 session_previous(struct session *);
+int		 session_next(struct session *, int);
+int		 session_previous(struct session *, int);
 int		 session_select(struct session *, int);
 int		 session_last(struct session *);
 
@@ -1467,6 +1513,12 @@ u_int	utf8_combine(const u_char *);
 void	utf8_split(u_int, u_char *);
 int	utf8_width(u_int);
 
+/* util.c */
+char   *section_string(char *, size_t, size_t, size_t);
+
+/* osdep-*.c */
+char   *get_argv0(int, char *);
+
 /* buffer.c */
 struct buffer 	*buffer_create(size_t);
 void		 buffer_destroy(struct buffer *);
