@@ -1,4 +1,4 @@
-/* $Id: tty-write.c,v 1.6 2009/01/14 19:29:32 nicm Exp $ */
+/* $Id: tty-write.c,v 1.9 2009/01/28 19:52:21 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -45,6 +45,8 @@ tty_vwrite_window(void *ptr, enum tty_cmd cmd, va_list ap)
 		c = ARRAY_ITEM(&clients, i);
 		if (c == NULL || c->session == NULL)
 			continue;
+		if (c->flags & CLIENT_SUSPENDED)
+			continue;
 
 		if (c->session->curw->window == wp->window) {
 			va_copy(aq, ap);
@@ -53,3 +55,25 @@ tty_vwrite_window(void *ptr, enum tty_cmd cmd, va_list ap)
 		}
 	}
 }
+
+void
+tty_write_update_mode(void *ptr, int mode)
+{
+	struct window_pane	*wp = ptr;
+	struct client		*c;
+	u_int		 	 i;
+
+	if (wp->window->flags & WINDOW_HIDDEN || wp->flags & PANE_HIDDEN)
+		return;
+
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c == NULL || c->session == NULL)
+			continue;
+		if (c->flags & CLIENT_SUSPENDED)
+			continue;
+
+		tty_update_mode(&c->tty, mode);
+	}
+}
+
