@@ -1,4 +1,4 @@
-/* $Id: tty.c,v 1.58 2009/01/11 23:31:46 nicm Exp $ */
+/* $Id: tty.c,v 1.65 2009/02/07 00:05:23 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -50,14 +50,11 @@ void	tty_cmd_clearline(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_clearscreen(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_clearstartofline(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_clearstartofscreen(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_cursormode(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_deletecharacter(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_deleteline(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_insertcharacter(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_insertline(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_insertmode(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_kcursormode(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_kkeypadmode(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_linefeed(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_mousemode(struct tty *, struct screen *, u_int, va_list);
 void	tty_cmd_reverseindex(struct tty *, struct screen *, u_int, va_list);
@@ -70,16 +67,11 @@ void (*tty_cmds[])(struct tty *, struct screen *, u_int, va_list) = {
 	tty_cmd_clearscreen,
 	tty_cmd_clearstartofline,
 	tty_cmd_clearstartofscreen,
-	tty_cmd_cursormode,
 	tty_cmd_deletecharacter,
 	tty_cmd_deleteline,
 	tty_cmd_insertcharacter,
 	tty_cmd_insertline,
-	tty_cmd_insertmode,
-	tty_cmd_kcursormode,
-	tty_cmd_kkeypadmode,
 	tty_cmd_linefeed,
-	tty_cmd_mousemode,
 	tty_cmd_reverseindex,
 };
 
@@ -98,10 +90,6 @@ tty_init(struct tty *tty, char *path, char *term)
 int
 tty_open(struct tty *tty, char **cause)
 {
-	struct termios	 tio;
-#ifdef TIOCFLUSH
-	int		 what;
-#endif
 	int		 mode;
 
 	tty->fd = open(tty->path, O_RDWR|O_NONBLOCK);
@@ -120,9 +108,6 @@ tty_open(struct tty *tty, char **cause)
 	else
 		tty->log_fd = -1;
 
-	tty->cx = UINT_MAX;
-	tty->cy = UINT_MAX;
-
 	if ((tty->term = tty_term_find(tty->termname, tty->fd, cause)) == NULL)
 		goto error;
 
@@ -130,7 +115,29 @@ tty_open(struct tty *tty, char **cause)
 	tty->out = buffer_create(BUFSIZ);
 
 	tty->flags &= TTY_UTF8;
-	memcpy(&tty->cell, &grid_default_cell, sizeof tty->cell);
+
+	tty_start_tty(tty);
+
+	tty_keys_init(tty);
+
+	tty_fill_acs(tty);
+
+	return (0);
+
+error:
+	close(tty->fd);
+	tty->fd = -1;
+
+	return (-1);
+}
+
+void
+tty_start_tty(struct tty *tty)
+{
+	struct termios	 tio;
+#ifdef TIOCFLUSH
+	int		 what;
+#endif
 
 	if (tcgetattr(tty->fd, &tty->tio) != 0)
 		fatal("tcgetattr failed");
@@ -159,18 +166,51 @@ tty_open(struct tty *tty, char **cause)
 	tty_putcode(tty, TTYC_SMKX);
 	tty_putcode(tty, TTYC_ENACS);
 	tty_putcode(tty, TTYC_CLEAR);
+	
+	tty_putcode(tty, TTYC_CNORM);
+	if (tty_term_has(tty->term, TTYC_KMOUS))
+		tty_puts(tty, "\033[?1000l");
 
-	tty_keys_init(tty);
+	memcpy(&tty->cell, &grid_default_cell, sizeof tty->cell);
 
-	tty_fill_acs(tty);
+	tty->cx = UINT_MAX;
+	tty->cy = UINT_MAX;
 
-	return (0);
+	tty->rlower = UINT_MAX;
+	tty->rupper = UINT_MAX;
 
-error:
-	close(tty->fd);
-	tty->fd = -1;
+	tty->mode = MODE_CURSOR;
+}
 
-	return (-1);
+void
+tty_stop_tty(struct tty *tty)
+{
+	struct winsize	ws;
+
+	/*
+	 * Be flexible about error handling and try not kill the server just
+	 * because the fd is invalid. Things like ssh -t can easily leave us
+	 * with a dead tty.
+	 */
+	if (ioctl(tty->fd, TIOCGWINSZ, &ws) == -1) {
+		if (errno != EBADF && errno != ENXIO && errno != ENOTTY)
+			fatal("ioctl(TIOCGWINSZ)");
+	} else if (tcsetattr(tty->fd, TCSANOW, &tty->tio) == -1) {
+		if (errno != EBADF && errno != ENXIO && errno != ENOTTY)
+			fatal("tcsetattr failed");
+	} else {
+		tty_raw(tty,
+		    tty_term_string2(tty->term, TTYC_CSR, 0, ws.ws_row - 1));
+		tty_raw(tty, tty_term_string(tty->term, TTYC_RMACS));
+		tty_raw(tty, tty_term_string(tty->term, TTYC_SGR0));
+		tty_raw(tty, tty_term_string(tty->term, TTYC_CLEAR));
+		tty_raw(tty, tty_term_string(tty->term, TTYC_RMKX));
+		tty_raw(tty, tty_term_string(tty->term, TTYC_RMCUP));
+
+		tty_raw(tty, tty_term_string(tty->term, TTYC_CNORM));
+		if (tty_term_has(tty->term, TTYC_KMOUS))
+			tty_raw(tty, "\033[?1000l");
+	}
 }
 
 void
@@ -198,10 +238,8 @@ tty_get_acs(struct tty *tty, u_char ch)
 }
 
 void
-tty_close(struct tty *tty)
+tty_close(struct tty *tty, int no_stop)
 {
-	struct winsize	ws;
-
 	if (tty->fd == -1)
 		return;
 
@@ -210,27 +248,8 @@ tty_close(struct tty *tty)
 		tty->log_fd = -1;
 	}
 
-	/*
-	 * Be flexible about error handling and try not kill the server just
-	 * because the fd is invalid. Things like ssh -t can easily leave us
-	 * with a dead tty.
-	 */
-	if (ioctl(tty->fd, TIOCGWINSZ, &ws) == -1) {
-		if (errno != EBADF && errno != ENXIO && errno != ENOTTY)
-			fatal("ioctl(TIOCGWINSZ)");
-	} else if (tcsetattr(tty->fd, TCSANOW, &tty->tio) == -1) {
-		if (errno != EBADF && errno != ENXIO && errno != ENOTTY)
-			fatal("tcsetattr failed");
-	} else {
-		tty_raw(tty,
-		    tty_term_string2(tty->term, TTYC_CSR, 0, ws.ws_row - 1));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_RMACS));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_SGR0));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_CLEAR));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_RMKX));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_RMCUP));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_CNORM));
-	}
+	if (!no_stop)
+		tty_stop_tty(tty);
 
 	tty_term_free(tty->term);
 	tty_keys_free(tty);
@@ -243,9 +262,9 @@ tty_close(struct tty *tty)
 }
 
 void
-tty_free(struct tty *tty)
+tty_free(struct tty *tty, int no_stop)
 {
-	tty_close(tty);
+	tty_close(tty, no_stop);
 
 	if (tty->path != NULL)
 		xfree(tty->path);
@@ -299,7 +318,8 @@ tty_putc(struct tty *tty, char ch)
 		ch = tty_get_acs(tty, ch);
 	buffer_write8(tty->out, ch);
 
-	tty->cx++;	/* This is right most of the time. */
+	if (ch >= 0x20)
+		tty->cx++;	/* This is right most of the time. */
 
 	if (tty->log_fd != -1)
 		write(tty->log_fd, &ch, 1);
@@ -318,6 +338,30 @@ tty_set_title(struct tty *tty, const char *title)
 	tty_putc(tty, '\007');
 }
 
+void
+tty_update_mode(struct tty *tty, int mode)
+{
+	int	changed;
+
+	if (tty->flags & TTY_NOCURSOR)
+		mode &= ~MODE_CURSOR;
+
+	changed = mode ^ tty->mode;
+	if (changed & MODE_CURSOR) {
+		if (mode & MODE_CURSOR)
+			tty_putcode(tty, TTYC_CNORM);
+		else
+			tty_putcode(tty, TTYC_CIVIS);
+	}
+	if (changed & MODE_MOUSE) {
+		if (mode & MODE_MOUSE)
+			tty_puts(tty, "\033[?1000h");
+		else
+			tty_puts(tty, "\033[?1000l");
+	}
+	tty->mode = mode;
+}
+
 void
 tty_emulate_repeat(
     struct tty *tty, enum tty_code_code code, enum tty_code_code code1, u_int n)
@@ -476,24 +520,6 @@ tty_cmd_clearstartofline(
 	}
 }
 
-void
-tty_cmd_cursormode(
-    struct tty *tty, unused struct screen *s, unused u_int oy, va_list ap)
-{
-	int	ua;
-
-	ua = va_arg(ap, int);
-
-	if (tty->cursor == ua)
-		return;
-	tty->cursor = ua;
-
-	if (ua && !(tty->flags & TTY_NOCURSOR))
-		tty_putcode(tty, TTYC_CNORM);
-	else
-		tty_putcode(tty, TTYC_CIVIS);
-}
-
 void
 tty_cmd_reverseindex(
     struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
@@ -503,53 +529,6 @@ tty_cmd_reverseindex(
 	tty_cursor(tty, s->cx, s->cy, oy);
 
 	tty_putcode(tty, TTYC_RI);
-	
-}
-
-void
-tty_cmd_insertmode(unused struct tty *tty,
-    unused struct screen *s, unused u_int oy, va_list ap)
-{
-	int	ua;
-
-	ua = va_arg(ap, int);
-
-#if 0
-	/* XXX */
-	if (ua)
-		tty_puts(tty, enter_insert_mode);
-	else
-		tty_puts(tty, exit_insert_mode);
-#endif
-}
-
-void
-tty_cmd_mousemode(
-    struct tty *tty, unused struct screen *s, unused u_int oy, va_list ap)
-{
-	int	ua;
-
-	if (!tty_term_has(tty->term, TTYC_KMOUS))
-		return;
-
-	ua = va_arg(ap, int);
-
-	if (ua)
-		tty_puts(tty, "\033[?1000h");
-	else
-		tty_puts(tty, "\033[?1000l");
-}
-
-void
-tty_cmd_kcursormode(unused struct tty *tty,
-    unused struct screen *s, unused u_int oy, unused va_list ap)
-{
-}
-
-void
-tty_cmd_kkeypadmode(unused struct tty *tty,
-    unused struct screen *s, unused u_int oy, unused va_list ap)
-{
 }
 
 void
@@ -560,6 +539,7 @@ tty_cmd_linefeed(struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
 	tty_cursor(tty, s->cx, s->cy, oy);
 
 	tty_putc(tty, '\n');
+	tty->cy++;
 }
 
 void
@@ -662,20 +642,18 @@ tty_cmd_cell(struct tty *tty, struct screen *s, u_int oy, va_list ap)
 	if (gc->flags & GRID_FLAG_PADDING)
 		return;
 
-	/* Handle special characters. Should never come into this function.*/
-	if (gc->data < 0x20 || gc->data == 0x7f)
-		return;
-
 	/* Set the attributes. */
 	tty_attributes(tty, gc);
 
-	/* If not UTF8 multibyte, write directly. */
-	if (gc->data <= 0xff) {
+	/* If not UTF-8, write directly. */
+	if (!(gc->flags & GRID_FLAG_UTF8)) {
+		if (gc->data > 0xff || gc->data < 0x20 || gc->data == 0x7f)
+			return;
 		tty_putc(tty, gc->data);
 		return;
 	}
 
-	/* If the terminal doesn't support UTF8, write _s. */
+	/* If the terminal doesn't support UTF-8, write underscores. */
 	if (!(tty->flags & TTY_UTF8)) {
 		width = utf8_width(gc->data);
 		while (width-- > 0)
@@ -683,7 +661,7 @@ tty_cmd_cell(struct tty *tty, struct screen *s, u_int oy, va_list ap)
 		return;
 	}
 
-	/* Unpack UTF-8 and write it. */
+	/* Otherwise, unpack UTF-8 and write it. */
 	utf8_split(gc->data, out);
 	for (i = 0; i < 4; i++) {
 		if (out[i] == 0xff)
@@ -722,7 +700,10 @@ tty_region(struct tty *tty, struct screen *s, u_int oy)
 void
 tty_cursor(struct tty *tty, u_int cx, u_int cy, u_int oy)
 {
-	if (tty->cx != cx || tty->cy != oy + cy) {
+	if (cx == 0 && tty->cx != 0 && tty->cy == oy + cy) {
+		tty->cx = 0;
+		tty_putc(tty, '\r');
+	} else if (tty->cx != cx || tty->cy != oy + cy) {
 		tty->cx = cx;
 		tty->cy = oy + cy;
 		tty_putcode2(tty, TTYC_CUP, tty->cy, tty->cx);
@@ -734,6 +715,7 @@ tty_attributes(struct tty *tty, const struct grid_cell *gc)
 {
 	struct grid_cell	*tc = &tty->cell;
 	u_char			 changed;
+	u_int			 fg, bg;
 
 	/* If any bits are being cleared, reset everything. */
 	if (tc->attr & ~gc->attr)
@@ -744,6 +726,8 @@ tty_attributes(struct tty *tty, const struct grid_cell *gc)
 	tc->attr = gc->attr;
 
 	/* Set the attributes. */
+	fg = gc->fg;
+	bg = gc->bg;
 	if (changed & GRID_ATTR_BRIGHT)
 		tty_putcode(tty, TTYC_BOLD);
 	if (changed & GRID_ATTR_DIM)
@@ -754,25 +738,29 @@ tty_attributes(struct tty *tty, const struct grid_cell *gc)
 		tty_putcode(tty, TTYC_SMUL);
 	if (changed & GRID_ATTR_BLINK)
 		tty_putcode(tty, TTYC_BLINK);
-	if (changed & GRID_ATTR_REVERSE)
-		tty_putcode(tty, TTYC_REV);
+	if (changed & GRID_ATTR_REVERSE) {
+		if (tty_term_has(tty->term, TTYC_REV))
+			tty_putcode(tty, TTYC_REV);
+		else if (tty_term_has(tty->term, TTYC_SMSO))
+			tty_putcode(tty, TTYC_SMSO);    
+	}
 	if (changed & GRID_ATTR_HIDDEN)
 		tty_putcode(tty, TTYC_INVIS);
 	if (changed & GRID_ATTR_CHARSET)
 		tty_putcode(tty, TTYC_SMACS);
 
 	/* Set foreground colour. */
-	if (gc->fg != tc->fg ||
+	if (fg != tc->fg ||
 	    (gc->flags & GRID_FLAG_FG256) != (tc->flags & GRID_FLAG_FG256)) {
 		tty_attributes_fg(tty, gc);
-		tc->fg = gc->fg;
+		tc->fg = fg;
 	}
 
 	/* Set background colour. */
-	if (gc->bg != tc->bg ||
+	if (bg != tc->bg ||
 	    (gc->flags & GRID_FLAG_BG256) != (tc->flags & GRID_FLAG_BG256)) {
 		tty_attributes_bg(tty, gc);
-		tc->bg = gc->bg;
+		tc->bg = bg;
 	}
 }
 
