@@ -1,4 +1,4 @@
-/* $Id: window-copy.c,v 1.39 2009/01/12 00:52:37 nicm Exp $ */
+/* $Id: window-copy.c,v 1.49 2009/01/28 22:00:22 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -26,6 +26,8 @@ struct screen *window_copy_init(struct window_pane *);
 void	window_copy_free(struct window_pane *);
 void	window_copy_resize(struct window_pane *, u_int, u_int);
 void	window_copy_key(struct window_pane *, struct client *, int);
+void	window_copy_mouse(
+    	    struct window_pane *, struct client *, u_char, u_char, u_char);
 
 void	window_copy_redraw_lines(struct window_pane *, u_int, u_int);
 void	window_copy_redraw_screen(struct window_pane *);
@@ -64,7 +66,8 @@ const struct window_mode window_copy_mode = {
 	window_copy_free,
 	window_copy_resize,
 	window_copy_key,
-	NULL
+	window_copy_mouse,
+	NULL,
 };
 
 struct window_copy_mode_data {
@@ -96,6 +99,8 @@ window_copy_init(struct window_pane *wp)
 
 	s = &data->screen;
 	screen_init(s, screen_size_x(&wp->base), screen_size_y(&wp->base), 0);
+	s->mode |= MODE_MOUSE;
+
 	s->cx = data->cx;
 	s->cy = data->cy;
 
@@ -117,6 +122,20 @@ window_copy_free(struct window_pane *wp)
 	xfree(data);
 }
 
+void
+window_copy_pageup(struct window_pane *wp)
+{
+	struct window_copy_mode_data	*data = wp->modedata;
+	struct screen			*s = &data->screen;
+	
+	if (data->oy + screen_size_y(s) > screen_hsize(&wp->base))
+		data->oy = screen_hsize(&wp->base);
+	else
+		data->oy += screen_size_y(s);
+	window_copy_update_selection(wp);
+	window_copy_redraw_screen(wp);
+}
+     
 void
 window_copy_resize(struct window_pane *wp, u_int sx, u_int sy)
 {
@@ -156,12 +175,7 @@ window_copy_key(struct window_pane *wp, struct client *c, int key)
 		window_copy_cursor_down(wp);
 		return;
 	case MODEKEY_PPAGE:
-		if (data->oy + screen_size_y(s) > screen_hsize(&wp->base))
-			data->oy = screen_hsize(&wp->base);
-		else
-			data->oy += screen_size_y(s);
-		window_copy_update_selection(wp);
-		window_copy_redraw_screen(wp);
+		window_copy_pageup(wp);
 		break;
 	case MODEKEY_NPAGE:
 		if (data->oy < screen_size_y(s))
@@ -172,13 +186,14 @@ window_copy_key(struct window_pane *wp, struct client *c, int key)
 		window_copy_redraw_screen(wp);
 		break;
 	case MODEKEY_STARTSEL:
-		window_copy_start_selection(wp);
+ 		window_copy_start_selection(wp);
 		break;
 	case MODEKEY_CLEARSEL:
 		screen_clear_selection(&data->screen);
 		window_copy_redraw_screen(wp);
 		break;
 	case MODEKEY_COPYSEL:
+	case MODEKEY_ENTER:
 		if (c != NULL && c->session != NULL) {
 			window_copy_copy_selection(wp, c);
 			window_pane_reset_mode(wp);
@@ -201,6 +216,28 @@ window_copy_key(struct window_pane *wp, struct client *c, int key)
 	}
 }
 
+void
+window_copy_mouse(struct window_pane *wp,
+    unused struct client *c, u_char b, u_char x, u_char y)
+{
+	struct window_copy_mode_data	*data = wp->modedata;
+	struct screen			*s = &data->screen;
+
+	if ((b & 3) == 3)
+		return;
+	if (x >= screen_size_x(s))
+		return;
+	if (y >= screen_size_y(s))
+		return;
+
+	data->cx = x;
+	data->cy = y;
+	
+	if (window_copy_update_selection(wp))
+ 		window_copy_redraw_screen(wp);		
+	window_copy_update_cursor(wp);
+}
+
 void
 window_copy_write_line(struct window_pane *wp, struct screen_write_ctx *ctx, u_int py)
 {
@@ -214,8 +251,9 @@ window_copy_write_line(struct window_pane *wp, struct screen_write_ctx *ctx, u_i
 		memcpy(&gc, &grid_default_cell, sizeof gc);
 		size = xsnprintf(hdr, sizeof hdr,
 		    "[%u,%u/%u]", data->ox, data->oy, screen_hsize(&wp->base));
-		gc.fg = options_get_number(&wp->window->options, "mode-fg");
-		gc.bg = options_get_number(&wp->window->options, "mode-bg");
+		gc.bg = options_get_number(&wp->window->options, "mode-fg");
+		gc.fg = options_get_number(&wp->window->options, "mode-bg");
+		gc.attr |= options_get_number(&wp->window->options, "mode-attr");
 		screen_write_cursormove(ctx, screen_size_x(s) - size, 0);
 		screen_write_puts(ctx, &gc, "%s", hdr);
 		screen_write_puts(ctx, &gc, "%s", hdr);
@@ -318,8 +356,9 @@ window_copy_update_selection(struct window_pane *wp)
 
 	/* Set colours. */
 	memcpy(&gc, &grid_default_cell, sizeof gc);
-	gc.fg = options_get_number(&wp->window->options, "mode-fg");
-	gc.bg = options_get_number(&wp->window->options, "mode-bg");
+	gc.bg = options_get_number(&wp->window->options, "mode-fg");
+	gc.fg = options_get_number(&wp->window->options, "mode-bg");
+	gc.attr |= options_get_number(&wp->window->options, "mode-attr");
 
 	/* Find top-left of screen. */
 	tx = data->ox;
@@ -401,11 +440,6 @@ window_copy_copy_selection(struct window_pane *wp, struct client *c)
 		xx = window_copy_find_length(wp, sy);
 		window_copy_copy_line(wp, &buf, &off, sy, sx, xx);
 		if (ey - sy > 1) {
-			/*
-			 * XXX gcc -O2 completely screws this up and either
-			 * causes a segfault or exits the loop after one
-			 * iteration.
-			 */
 			for (i = sy + 1; i < ey - 1; i++) {
 				xx = window_copy_find_length(wp, i);
 				window_copy_copy_line(wp, &buf, &off, i, 0, xx);
@@ -421,7 +455,6 @@ window_copy_copy_selection(struct window_pane *wp, struct client *c)
 	/* Add the buffer to the stack. */
 	limit = options_get_number(&c->session->options, "buffer-limit");
 	paste_add(&c->session->buffers, buf, limit);
-	xfree(buf);
 }
 
 void
@@ -429,7 +462,8 @@ window_copy_copy_line(
     struct window_pane *wp, char **buf, size_t *off, u_int sy, u_int sx, u_int ex)
 {
 	const struct grid_cell *gc;
-	u_char			i, j, xx, data[4];
+	u_int			i, j, xx;
+	u_char			data[4];
 
 	if (sx > ex)
 		return;
