@@ -1,4 +1,4 @@
-/* $Id: window-scroll.c,v 1.27 2009/01/11 23:31:46 nicm Exp $ */
+/* $Id: window-scroll.c,v 1.31 2009/01/28 19:52:21 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -43,7 +43,8 @@ const struct window_mode window_scroll_mode = {
 	window_scroll_free,
 	window_scroll_resize,
 	window_scroll_key,
-	NULL
+	NULL,
+	NULL,
 };
 
 struct window_scroll_mode_data {
@@ -86,6 +87,20 @@ window_scroll_free(struct window_pane *wp)
 	xfree(data);
 }
 
+void
+window_scroll_pageup(struct window_pane *wp)
+{
+	struct window_scroll_mode_data	*data = wp->modedata;
+	struct screen			*s = &data->screen;
+	
+	if (data->oy + screen_size_y(s) > screen_hsize(&wp->base))
+		data->oy = screen_hsize(&wp->base);
+	else
+		data->oy += screen_size_y(s);
+	
+	window_scroll_redraw_screen(wp);
+}
+
 void
 window_scroll_resize(struct window_pane *wp, u_int sx, u_int sy)
 {
@@ -126,11 +141,7 @@ window_scroll_key(struct window_pane *wp, unused struct client *c, int key)
 		window_scroll_scroll_down(wp);
 		break;
 	case MODEKEY_PPAGE:
-		if (data->oy + screen_size_y(s) > screen_hsize(&wp->base))
-			data->oy = screen_hsize(&wp->base);
-		else
-			data->oy += screen_size_y(s);
-		window_scroll_redraw_screen(wp);
+		window_scroll_pageup(wp);
 		break;
 	case MODEKEY_NPAGE:
 		if (data->oy < screen_size_y(s))
@@ -158,8 +169,9 @@ window_scroll_write_line(
 		memcpy(&gc, &grid_default_cell, sizeof gc);
 		size = xsnprintf(hdr, sizeof hdr,
 		    "[%u,%u/%u]", data->ox, data->oy, screen_hsize(&wp->base));
-		gc.fg = options_get_number(&wp->window->options, "mode-fg");
-		gc.bg = options_get_number(&wp->window->options, "mode-bg");
+		gc.bg = options_get_number(&wp->window->options, "mode-fg");
+		gc.fg = options_get_number(&wp->window->options, "mode-bg");
+		gc.attr |= options_get_number(&wp->window->options, "mode-attr");
 		screen_write_cursormove(ctx, screen_size_x(s) - size, 0);
 		screen_write_puts(ctx, &gc, "%s", hdr);
 		memcpy(&gc, &grid_default_cell, sizeof gc);
